/***************************************************************************
                          cmdprefs.cpp  -  description
                             -------------------
    begin                : Fri Feb 21 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cmdprefs.h"
#include "definitions.h"
#include "lconfig.h"
#include "eakprocs.h"
#include "msgpasser.h"

#include <unistd.h>

extern "C" {
 #ifdef HAVE_GETOPT_H
 #  define HAVE_GETOPT_LONG
 #  include <getopt.h>
 #endif
}

CmdPrefs::CmdPrefs(){
	           cdrom = null;
	           mixer = null;
	           kbtype = null;
	           background = false;
	           verbose = false;
	           conffile = null;
	           deffile = null;
	           userdeffile = null;
	           showusage = false;
	           createnewconfig = false;
	           showkeyboards = false;
               #ifdef HAVE_XOSD
               /** Set the XOSD values to lineakd defaults */
               XOSD_font = null;
               XOSD_color = null;
               XOSD_timeout = -1;
               XOSD_pos = XOSD_POS;
               XOSD_align =XOSD_ALIGN;
               XOSD_pos_set = false;
               XOSD_align_set = false;
               XOSD_hoffset = -1;
               XOSD_voffset = -1;
               XOSD_soffset = -1;
               #endif

}
CmdPrefs::~CmdPrefs(){
}
/** Read property of string  conffile. */
const string & CmdPrefs::getConffile(){
	return conffile;
}
/** Read property of string  deffile. */
const string & CmdPrefs::getDeffile(){
	return deffile;
}
const string & CmdPrefs::getUserDeffile(){
	return userdeffile;
}
/** Read property of bool background. */
const bool& CmdPrefs::getBackground(){
	return background;
}
/** Read property of bool verbose. */
const bool& CmdPrefs::getVerbose(){
	return verbose;
}
/** Read property of string  cdrom. */
const string & CmdPrefs::getCdrom(){
	return cdrom;
}
/** Read property of string  mixer. */
const string & CmdPrefs::getMixer(){
	return mixer;
}
/** Read property of string  kbtype. */
const string & CmdPrefs::getKbtype(){
	return kbtype;
}
#ifdef HAVE_XOSD
const string & CmdPrefs::getXOSD_font() {
    return XOSD_font;
}
const string & CmdPrefs::getXOSD_color() {
    return XOSD_color;
}
const int & CmdPrefs::getXOSD_timeout() {
    return XOSD_timeout;
}
const xosd_pos & CmdPrefs::getXOSD_pos() {
    return XOSD_pos;
}
const xosd_align & CmdPrefs::getXOSD_align() {
    return XOSD_align;
}
const int & CmdPrefs::getXOSD_hoffset() {
    return XOSD_hoffset;
}
const int & CmdPrefs::getXOSD_voffset() {
    return XOSD_voffset;
}
const int & CmdPrefs::getXOSD_soffset() {
    return XOSD_soffset;
}
#endif

bool CmdPrefs::showUsage() {
	return showusage;
}
bool CmdPrefs::createNewConfig() {
	return createnewconfig;
}
bool CmdPrefs::showKeyboards() {
	return showkeyboards;
}
/** Set the global options in the keyboard config.
      Command line options override config file options. */
void CmdPrefs::setOpts( LConfig & conf){
	if (cdrom != null)
		conf.cdrom = cdrom;
	if (kbtype != null)
		conf.kbdtype = kbtype;
	if (mixer != null)
		conf.mixer = mixer;

    #ifdef HAVE_XOSD
    if (XOSD_font != null)
      conf.XOSD_font = XOSD_font;
    if (XOSD_color != null)
      conf.XOSD_color = XOSD_color;
    if (XOSD_timeout != -1)
      conf.XOSD_timeout = XOSD_timeout;
    /** Must use a flag to see if it was set b/c the types are enums */
    if (XOSD_pos_set)  
      conf.XOSD_pos = XOSD_pos;
    if (XOSD_align_set)
      conf.XOSD_align = XOSD_align;
      
    if (XOSD_hoffset != -1)
      conf.XOSD_hoffset = XOSD_hoffset;
    if (XOSD_voffset != -1)
      conf.XOSD_voffset = XOSD_voffset;
    if (XOSD_soffset != -1)
      conf.XOSD_soffset = XOSD_soffset;
    #endif

    
}
bool CmdPrefs::parseArgs(int argc, char *argv[]) {

	using namespace lineak_core_functions;
	int myopt;
	bool newconf=false;
    #ifdef HAVE_XOSD
    string position;
    string align;
    const char *flags = "hd:m:c:bvrxf:e:u:ln:o:t:p:a:y:w:s:";
    #else
	const char *flags = "hd:m:c:bvrxf:e:u:l";
    #endif
    
#ifdef HAVE_GETOPT_LONG
  	int option_index = 0;
  	static struct option long_options[] = {
        {"help",             0, 0, 'h'},
        {"cdrom-dev",    1, 0, 'd'},
        {"mixer-dev",     1, 0, 'm'},
        {"create",            1, 0, 'c'},
        {"verbose",         0, 0, 'v'},
        {"kbd-list",         0, 0, 'l'},
        {"conffile",          1, 0, 'f'},
        {"deffile",            1, 0, 'e'},
        {"userdeffile",     1,0,'u'},
        {"verbose",          0,0, 'v'},
        {"reload",            0,0, 'r'},
        {"exit",                0,0, 'x'},
        #ifdef HAVE_XOSD
        {"xosd-font",        1, 0, 'n'},
        {"xosd-color",       1, 0, 'o'},
        {"xosd-timeout",  1, 0, 't'},
        {"xosd-pos",         1, 0, 'p'},
        {"xosd-align",      1, 0, 'a'},
        {"xosd-hoffset",   1, 0, 'y'},
        {"xosd-voffset",   1, 0, 'w'},
        {"xosd-soffset",    1, 0, 's'},
        #endif
        {0, 0, 0, 0}
  	};
  	while ((myopt = getopt_long(argc, argv, flags, long_options, &option_index)) != -1) {
#else
  	while ((myopt = getopt(argc, argv, flags)) != -1) {
#endif
	/* parse each option */
    switch (myopt) {
          case 'd':
			  if (newconf) { // -d parameter has to appear BEFORE -c
				cerr << "*** -d parameter ignored, must be specified BEFORE -c" << endl;
				break;
			  }
		      cdrom = optarg;
              break;
          case 'm':
			  if (newconf) { // -m parameter has to appear BEFORE -c
				cerr << "*** -m parameter ignored, must be specified BEFORE -c" << endl;
				break;
			  }
			  mixer = optarg;
              break;
          case 'e':
			  if (newconf) { // -e parameter has to appear BEFORE -c
				cerr << "*** -e parameter ignored, must be specified BEFORE -c" << endl;
				break;
			  }
			  deffile = optarg;
              break;
          case 'u':
			  if (newconf) { // -e parameter has to appear BEFORE -c
				cerr << "*** -u parameter ignored, must be specified BEFORE -c" << endl;
				break;
			  }
			  userdeffile = optarg;
              break;
          case 'f':
			  if (newconf) { // -f parameter has to appear BEFORE -c
				cerr << "*** -f parameter ignored, must be specified BEFORE -c" << endl;;
				break;
			  }
			  conffile = optarg;
              break;
          case 'h':
              showusage = true;
              break;
          case 'c':
              cout << endl << "NOTE: if '-f', '-e','u', '-d', or '-m' parameters are used in conjunction with '-c', please\n      make sure you specify those BEFORE '-c' !\n" << endl;
		      kbtype = optarg;
              createnewconfig = true; 
  		      newconf = true;
              break;
          case 'v':
              verbose = true;
              break;
          case 'r':
          	{
              if (lineak_core_functions::is_running("lineakd")) {
          	  	   	msgPasser message;
                  if (verbose)
                  		cout << "Sending restart message\n";
                 	message.sendMessage(HUP,"hup");
              }
              exit(true);
              break;
             }
          case 'x':
          	{
              if (lineak_core_functions::is_running("lineakd")) {
          	  		msgPasser message;
                  if (verbose)
                  		cout << "Sending exit message\n";
              		message.sendMessage(EXIT,"exit");
              }
              exit(true);
              break;
             }
          case 'l':
              showkeyboards = true;
              break;
          #ifdef HAVE_XOSD
          case 'n':
              XOSD_font = optarg;
            break;
          case 'o':
              XOSD_color = optarg;
            break;
          case 't':
              XOSD_timeout = atoi(optarg);
            break;
          case 'p':
              position = optarg;
              if (position == "top") {
                XOSD_pos = XOSD_top;
                XOSD_pos_set = true;
              }
              if (position == "middle") {
                XOSD_pos = XOSD_middle;
                XOSD_pos_set = true;
              }
              if (position == "bottom") {
                XOSD_pos = XOSD_bottom;
                XOSD_pos_set = true;
              }
            break;
          case 'a':
              align = optarg;
              if (align == "left") {
                XOSD_align = XOSD_left;
                XOSD_align_set = true;
              }
              if (align == "center") {
                XOSD_align = XOSD_center;
                XOSD_align_set = true;
              }
              if (align == "right") {
                XOSD_align = XOSD_right;
                XOSD_align_set = true;
              }
            break;
          case 'y':
              XOSD_hoffset = atoi(optarg);
            break;
          case 'w':
              XOSD_voffset = atoi(optarg);
            break;
          case 's':
              XOSD_soffset = atoi(optarg);
            break;
          #endif                          
          default:
          	cout << "Incorrect arguments" << endl;
          	exit(false);
           break;
    }
  }

  /* check for a single additional argument */
  if ((argc - optind) > 1) {
    cerr <<  argv[0] << ": too many arguments" << endl;
	return false;
  }

  return true;
}
