/***************************************************************************
                          cmdprefs.h  -  description
                             -------------------
    begin                : Fri Feb 21 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CMDPREFS_H
#define CMDPREFS_H
#include "../config.h"
#ifdef  HAVE_XOSD

extern "C" {
#include <xosd.h>
}
#endif

#include <string>
using namespace std;
class LConfig;
/**Class to set and store command line preferences.
  *@author Sheldon Lee Wen
  */

class CmdPrefs {
public: 
	CmdPrefs();
	~CmdPrefs();
  	/** Read property of string  deffile. */
  	const string & getConffile();
  	/** Read property of string  deffile. */
  	const string & getDeffile();
    /** Read property of the user deffile */
   	const string & getUserDeffile();
  	/** Read property of bool background. */
  	const bool& getBackground();
  	/** Read property of bool verbose. */
  	const bool& getVerbose();
  	/** Read property of string  cdrom. */
  	const string & getCdrom();
  	/** Read property of string  mixer. */
  	const string & getMixer();
  	/** Read property of string  deffile. */
  	const string & getKbtype();
    #ifdef HAVE_XOSD
    /** accessors for all of the xosd variables we expose to the user */
    const string & getXOSD_font(void);
    const string & getXOSD_color();
    const int & getXOSD_timeout();
    const xosd_pos & getXOSD_pos();
    const xosd_align & getXOSD_align();
    const int & getXOSD_hoffset();
    const int & getXOSD_voffset();
    const int & getXOSD_soffset();
    #endif
  	/** Set the command line options on the config object. */
  	void setOpts( LConfig & conf);
   	/** Parse the command line args, and set the options */
    bool parseArgs(int argc, char *argv[]);
    /** return values of the action flags */
    bool showUsage();
    bool createNewConfig();
    bool showKeyboards();

private: // Public attributes
  /** CD-ROM device to use, passed in from the command line. */
  string cdrom;
  /** Mixer device to use, passed in from the command line. */
  string mixer;
  /** Keyboard type, passed in from the command line. */
  string kbtype;
  /** Run in background mode? Set from command line. */
  bool background;
  /** Run in verbose mode? Set from command line. */
  bool verbose;
  /** Configuration file to use, passed in from the command line. */
  string conffile;
  /** Definition file to use, passed in from the command line. */
  string deffile;
  /** User definition file to use, passed in from the command line. */
  string userdeffile;
  /** Possible actions to take from parsing the command line options. */
  bool showusage;
  bool createnewconfig;
  bool showkeyboards;
  
  #ifdef  HAVE_XOSD
  /** XOSD Settings */
  string XOSD_font;
  string XOSD_color;
  int XOSD_timeout;
  xosd_pos XOSD_pos;
  xosd_align XOSD_align;
  int XOSD_hoffset;
  int XOSD_voffset;
  int XOSD_soffset;
  bool XOSD_pos_set, XOSD_align_set;
  
  #endif
};

#endif
