/***************************************************************************
                          commandexec.cpp  -  description
                             -------------------
    begin                : ?  8? 19 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../config.h"

extern "C" {
#include <sys/types.h>
#include <unistd.h>
}
#include <algorithm>
#include <cctype>

#include "commandexec.h"
#include "lkbd.h"
#include "lconfig.h"
#include "soundctrl.h"
#include "cdromctrl.h"
#include "lcommand.h"
#include "functions.h"


#ifdef HAVE_XOSD
CommandExec::CommandExec(LKbd & imyKbd, LConfig & imyConfig, xosdCtrl &imyXosd, const bool iverbose) :
            myKbd(imyKbd), myConfig(imyConfig), myXosd(imyXosd), verbose(iverbose)
{
		init_macros();
//		enabled = true;
}
#else
CommandExec::CommandExec(LKbd & imyKbd, LConfig & imyConfig, const bool iverbose) :
            myKbd(imyKbd), myConfig(imyConfig), verbose(iverbose)
{
//		enabled = true;
		init_macros();
}
#endif

void CommandExec::init_macros() {
    specials.push_back(EAK_EJECT);
    specials.push_back(EAK_VOLUP);
    specials.push_back(EAK_VOLDOWN);
    specials.push_back(EAK_MUTE);
    specials.push_back(EAK_SLEEP);
    specials.push_back(EAK_SCREEN_LOCK);
}
CommandExec::~CommandExec(){
}
/** Execute the command for the keycode. */
void CommandExec::exec(int keycode){
//		if (enabled) {
		LCommand command = myKbd.getKey(keycode).getCommand();
		string dname = myKbd.getKey(keycode).getDisplayName();
		#ifdef _DEBUG_
	  cout << "Display: " << dname << endl;	
		cout << "For command: " << endl;
		cout << command << endl;
		cout << "--------------" << endl;
		#endif
  	/* lookup the key in our EAKeylist */
  	if (command.isEmpty())    {
		/* oops, not found... how did THAT happen? */
		cerr << "Key with keycode " << keycode << " not found!!" << endl;
        cout << myKbd.getKey(keycode);
        cout << myKbd.getKey(keycode).getCommand();
  	}
  	else {
			if (verbose) cout << "... that's the " <<  myKbd.getKey(keycode).getKeyname() << " key" << endl;
			/* check if the key has to run a SPECIAL command */
			if ( command.getMacroType() == EAK_EJECT) { /* Eject */
    	  macroEAK_EJECT(command);
			} else if ( command.getMacroType() == EAK_VOLUP) { /* VolumeUp */
    	  macroEAK_VOLUP(command);
			} else if (command.getMacroType() == EAK_VOLDOWN) { /* VolumeDown */
    	  macroEAK_VOLDOWN(command);
			} else if (command.getMacroType() == EAK_MUTE) { /* Mute */
    	  macroEAK_MUTE(command);
			} else if (command.getMacroType() == EAK_SLEEP) { /* Sleep (not implemented yet) */
    	  macroEAK_SLEEP(command);
			} else if (command.getMacroType() == EAK_SCREEN_LOCK) { /* Screen lock (not implemented yet) */
    	  macroEAK_SCREEN_LOCK(command);
    	} else if (command.getCommand() == null) { /* no command defined */
	  		if (verbose) cout << "... but it has no command bound to it :(" << endl;
	  	 	cout << myKbd.getKey(keycode);
			} else {
	    
        #ifdef HAVE_XOSD
		myXosd.show(dname);
        #endif

	    	if (!fork()) {
					string comm = lineak_util_functions::unescape(command.getCommand(), "\"#");
          //comm = lineak_util_functions::unescape(comm,"#");
					comm += " &";
				    /* child process that tries to run the command, and then exits */
	  		        /* all specials done, let's go for it... ;) */
	  		        if (verbose) cout << "... running " << comm << endl;
					system(comm.c_str());
					exit(1);
	  		}
	  		/* NOTE: no, we don't wait() for the child. we just ignore the SIGCLD signal */
			}
  	}
  	/** Rotate the key state if it's a toggleable key */
   	myKbd.getKey(keycode).toggleKeyState();
//    }
//    else
//    	cerr << "Lineakd command processing is currently disabled" << endl;
}
void CommandExec::macroEAK_EJECT(LCommand &command) {
	const vector<string>& args = command.getArgs();
  /** If there is just the macro, and no arguments, use the globally defined cdrom device, and
      eject that */
	if (args.size() == 0) {
        cdromCtrl myCdrom(myConfig.cdrom);
		myCdrom.eject();
    }
	/** If however, the macro contains arguments, then attempt to eject all of those */
	else {
	  cdromCtrl cd;
  	  for (vector<string>::const_iterator it = args.begin(); it != args.end(); it++) {
  		cd.setCdromdev(*it);
    	/** Output the command */
		#ifdef HAVE_XOSD
    		myXosd.show(string("Ejecting CDRom"));
  		#endif
   		cd.eject();
    }
  } 
}
/** macroEAK_Mute. In order to support using lots of soundCtrl devices in s disposable
	manner. i.e. create one when you want, then throw it away when you don't, and supporting
	lots of user mixer devices, we need to keep track of soundCtrl objects for each mixer we
	may have used. They get added on a per use basis. This is b/c the toggleMute() method of
	the soundCtrl class depends on a private variable that tracks the state of the mixer (i.e. muted
	or not). If we just instantiate and destroy the objects as we need them we loose the
	state of this variable. I support I could have created a static vector of mute states for the
	class, but I felt that was getting really beyond the function of this class. */
void CommandExec::macroEAK_MUTE(LCommand &command) {
	const vector<string>& args = command.getArgs();
  static map<string,soundCtrl> soundDevices;
	int retval = 0;
    
  /** If there is just the macro, and no arguments, use the default volume stepping, and
      adjust by that */
  if (args.size() == 0) {
      /* Set the mixer device */
     /** If the soundDevice object for the corresponding mixer is not in the map. Add it */
      if ((soundDevices[myConfig.mixer]).getMixer() != myConfig.mixer) {
  	    soundCtrl mySound(myConfig.mixer);
        soundDevices[myConfig.mixer] = mySound;
      }        
      retval = (soundDevices[myConfig.mixer]).toggleMute();
      #ifdef HAVE_XOSD
    	myXosd.volume(retval);
      #endif
  }
  else {
  /** We should have either an integer, or integer-string pairs, which correspond to the increment
  	value, and the device to adjust. */
    // Assume we have just an increment value.
    int value = 0;
  	if (args.size() == 1) {
   	  value = atoi(args[0].c_str());
     /** If the soundDevice object for the corresponding mixer is not in the map. Add it */
      if ((soundDevices[myConfig.mixer]).getMixer() != myConfig.mixer) {
  	    soundCtrl mySound(myConfig.mixer);
        soundDevices[myConfig.mixer] = mySound;
      }
      retval = (soundDevices[myConfig.mixer]).toggleMute(value);
      #ifdef HAVE_XOSD
  	    myXosd.volume(retval);
      #endif
    }
    // Assume we have integer and value pairs.
    else {
    	if (args.size() >=2 && (args.size() % 2 == 0)) {
//    		soundCtrl sound;
  			for (vector<string>::const_iterator it = args.begin(); it != args.end();) {
    			value = atoi(it->c_str());
      		    it++;
      		    string device = *it;
       		    it++;
                /** If there is no soundCtrl object for this mixer, add one */
                if ((soundDevices[device]).getMixer() != device) {
  	                soundCtrl mySound(device);
                    soundDevices[device] = mySound;
                }
      	        
        	    retval = (soundDevices[myConfig.mixer]).toggleMute(value);
		    	#ifdef HAVE_XOSD
  				  myXosd.volume(retval);
    			#endif
      	}
      }
    }
  }
}
void CommandExec::macroEAK_VOLUP(LCommand &command) {
	const vector<string>& args = command.getArgs();
	int retval = 0;
  /** If there is just the macro, and no arguments, use the default volume stepping, and
      adjust by that */
//      cout << "macroEAK_VOLUP" << endl;
//      cout << command << endl;
//      lineak_util_functions::inspect_vector(args);
    
	if (args.size() == 0) {
	if (verbose) cout << "doing default volume up\n";
  	    /* Set the mixer device */
  	    soundCtrl mySound(myConfig.mixer);
        retval = mySound.volumeUp();
    #ifdef HAVE_XOSD
    	myXosd.volume(retval);
    #endif
  }
  else {
  /** We should have either an integer, or integer-string pairs, which correspond to the increment
  	value, and the device to adjust. */
    // Assume we have just an increment value.
    int value = 0;
  	if (args.size() == 1) {
      if (verbose) cout << "single volume up\n";
      value = atoi(args[0].c_str());
	  soundCtrl mySound(myConfig.mixer);
      retval = mySound.volumeUp(value);
      #ifdef HAVE_XOSD
  	    myXosd.volume(retval);
      #endif
    }
    // Assume we have integer and value pairs.
    else {
    	if (args.size() >=2 && (args.size() % 2 == 0)) {

      	soundCtrl sound;
  		for (vector<string>::const_iterator it = args.begin(); it != args.end();) {
    		value = atoi(it->c_str());
      		it++;
      		string device = *it;
       		it++;
            if (verbose) cout << device << " adjusted by: " << value << endl;
        	sound.setMixer(device);
        	retval = sound.volumeUp(value);
		    #ifdef HAVE_XOSD
              myXosd.volume(retval);
    		#endif
      	}
      }
    }
  } 
}
void CommandExec::macroEAK_VOLDOWN(LCommand &command) {
	const vector<string>& args = command.getArgs();
	int retval = 0;
  /** If there is just the macro, and no arguments, use the default volume stepping, and
      adjust by that */

//      cout << "macroEAK_VOLDOWN" << endl;
//      cout << command << endl;
//      lineak_util_functions::inspect_vector(args);
      
	if (args.size() == 0) {
	  if (verbose) cout << "default volumeDown\n";
  	  /* Set the mixer device */
  	  soundCtrl mySound(myConfig.mixer);
  	  retval = mySound.volumeDown();
      #ifdef HAVE_XOSD
    	  myXosd.volume(retval);
      #endif
    }
    else {
    /** We should have either an integer, or integer-string pairs, which correspond to the increment
  	  value, and the device to adjust. */
      // Assume we have just an increment value.
      int value = 0;
  	  if (args.size() == 1) {
   			value = atoi(args[0].c_str());
      	// If the value is positive, make it negative.
        if ( value > 0)
          value = 0 - value;
     	if (verbose) cout << "single volume down by: " << value << endl;
  	    soundCtrl mySound(myConfig.mixer);
        retval = mySound.volumeDown(value);
        #ifdef HAVE_XOSD
	  	  	myXosd.volume(retval);
        #endif
    }
    // Assume we have integer and value pairs.
    else {
    	if (args.size() >=2 && (args.size() % 2 == 0)) {
    		soundCtrl sound;
  			for (vector<string>::const_iterator it = args.begin(); it != args.end();) {
    			value = atoi(it->c_str());
      		it++;
      		string device = *it;
       		it++;
	        if ( value > 0)
    	      value = 0 - value;
         	if (verbose) cout << device << " adjusted by: " << value << endl;
        	sound.setMixer(device);
        	retval = sound.volumeDown(value);
	    		#ifdef HAVE_XOSD
  						myXosd.volume(retval);
    			#endif
      	}
      }
    }
  }
}
void CommandExec::macroEAK_SLEEP(LCommand &command) {
	#ifdef HAVE_XOSD
  	  myXosd.show ("Sleep");
    #endif
}
void CommandExec::macroEAK_SCREEN_LOCK(LCommand &command) {
	const vector<string>& args = command.getArgs();
	if (!args.empty()) {
    string method = args[0];
    // Transform the method to uppercase so we can do a comparison.
    int (*pf)(int)=toupper; 
    std::transform(method.begin(), method.end(), method.begin(), pf);
		string comm;
	    	if (!fork()) {
	     		if (method == "KDE")
	     			comm = "dcop kdesktop KScreensaverIface lock";
	        if (method == "GNOME" || method == "XSCREENSAVER" )
	     			comm = "xscreensaver-command -lock";
	        
          //comm = lineak_util_functions::unescape(comm,"#");
					comm += " &";
			    /* child process that tries to run the command, and then exits */
	        /* all specials done, let's go for it... ;) */
	        if (verbose) cout << "Locking screen for desktop " << method << endl;
					system(comm.c_str());
					exit(1);
	  		}
	  		/* NOTE: no, we don't wait() for the child. we just ignore the SIGCLD signal */
	}
	else
		cerr << "EAK_SCREEN_LOCK macro requires an argument" << endl;
		
}
