/***************************************************************************
                          commandexec.h  -  description
                             -------------------
    begin                : ?  8? 19 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef COMMANDEXEC_H
#define COMMANDEXEC_H

#include "../config.h"
#include "definitions.h"

#ifdef HAVE_XOSD
#include "xosdctrl.h"
#endif

#include <string>
#include <vector>
using namespace std;

class LKbd;
class LConfig;
class soundCtrl;
class cdromCtrl;
class LCommand;


/**Execute a command for a given key.
  *@author Sheldon Lee Wen
  */

class CommandExec {
public: 
#ifdef HAVE_XOSD
  CommandExec(LKbd & imyKbd, LConfig & imyConfig, xosdCtrl &myXosd, const bool iverbose=false);
#else
  CommandExec(LKbd & imyKbd, LConfig & imyConfig, const bool iverbose=false);
#endif
	~CommandExec();
  /** Execute the command for the keycode. */
  void exec(int keycode);
  /** Perform the EAK_EJECT macro */
  void macroEAK_EJECT(LCommand &command);
  /** Decrease the volume */
  void macroEAK_VOLUP(LCommand &command);
  /** Increate the volume */
  void macroEAK_VOLDOWN(LCommand &command);
  /** macroEAK_Mute. In order to support using lots of soundCtrl devices in s disposable
	manner. i.e. create one when you want, then throw it away when you don't, and supporting
	lots of user mixer devices, we need to keep track of soundCtrl objects for each mixer we
	may have used. They get added on a per use basis. This is b/c the toggleMute() method of
	the soundCtrl class depends on a private variable that tracks the state of the mixer (i.e. muted
	or not). If we just instantiate and destroy the objects as we need them we loose the
	state of this variable. I support I could have created a static vector of mute states for the
	class, but I felt that was getting really beyond the function of this class. */
  void macroEAK_MUTE(LCommand &command);
	/** Currently, this does nothing. I'll implement it for kernel 2.6 where we have suspend to disk
		capabilities. */
  void macroEAK_SLEEP(LCommand &command);
  void macroEAK_SCREEN_LOCK(LCommand &command);
//  inline bool isEnabled() { return enabled; }
//  inline void setEnabled(bool _enabled) { enabled = _enabled; }
private: // Private methods
  LKbd& myKbd;
  LConfig& myConfig;
//  soundCtrl& mySound;
//  cdromCtrl& myCdrom;
  #ifdef HAVE_XOSD
  xosdCtrl& myXosd;
  #endif
  bool verbose;
//  /** Determine if the command processing is enabled or disabled */
//  bool enabled;
  /** Contains a list of the special actions */
  vector<string> specials;
  void init_macros();
};

#endif
