/***************************************************************************
                          eakprocs.cpp  -  description
                             -------------------
    begin                : Mon Feb 10 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "eakprocs.h"

#include <fstream>
#include <iostream>
#include <sstream>
#include <cstdlib>

extern "C" {
#ifdef HAVE_GETOPT_H
#  define HAVE_GETOPT_LONG
#  include <getopt.h>
#endif
#ifndef X_NOT_STDC_ENV
#	include <stdlib.h>
#else
	extern char *getenv();
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <signal.h>
#include <pthread.h>
#include <semaphore.h>
}

#include "definitions.h"
#include "saver.h"
#include "configloader.h"
#include "functions.h"
#include "defloader.h"
#include "ldef.h"
#include "xmgr.h"
#include "cmdprefs.h"
#include "soundctrl.h"
#include "cdromctrl.h"
#include "lkbd.h"
#include "lkey.h"
#include "msgpasser.h"
#include "lockctrl.h"
#include "lcommand.h"
#include "commandexec.h"

using namespace lineak_util_functions;
using namespace std;

extern LKbd myKbd;
extern LConfig myConfig;
extern LDef myDef;
extern Xmgr *X;
extern CmdPrefs cmdprefs;
extern CommandExec *exec;


#ifdef HAVE_XOSD_H
#include "xosdctrl.h"
extern xosdCtrl *myXosd;
#endif
//char message[] = "Bye";


/** Thread for the message queue */
pthread_t m_thread;
sem_t x_sem, hup_sem;
bool doneexit = false;

/* parse .conf file */
void lineak_core_functions::parseconffile(CmdPrefs & prefs, LConfig & myConfig) {
    string conffilename;
    string homedir = getenv("HOME");
  	/* set (globally) full path of config file ($HOME/.lineak/CONFIGFILE) */
  	if (prefs.getConffile() == null) {
   		if (!dir_exists(homedir + LINEAKDIR))
     		create_homedir();
    	conffilename = homedir + CONFFILE;
  	} else
   		conffilename = prefs.getConffile();

	/* does the file exist? */
  	if (!file_exists(conffilename)) {
		conffilename = SYS_CONFFILE;
		if (!file_exists(conffilename)) {
	   		/* file does not exist */
			cerr << "*** A configuration file was not found! ***" << endl;
			cerr << "       Please run lineakd -l to list the supported keyboards." << endl;
			cerr << "       Then run \"lineakd -c TYPE\" to create a configuration file for your keyboard." << endl;
			cerr << "       Once that is done, edit " << (homedir + CONFFILE) << " or " <<  SYS_CONFFILE << " to map your keys to commands, then run lineakd again." << endl;
	   		cleanexit(false);
		}
  	}
	ConfigLoader configldr(conffilename);
    configldr >> myConfig;
    
	if (myConfig.isEmpty()) {
		cerr << "*** Configuration file " << conffilename << " could not be loaded" << endl;
		cerr << "		Have you defined actions for your keys?" << endl;
		cleanexit(false);
	}
}

/* parse keyboard definition file */
void lineak_core_functions::parsedeffile(CmdPrefs & prefs, LDef & def) {
    string usrdeffilename = prefs.getUserDeffile(); //userdefconf;
    string deffilename = prefs.getDeffile(); //defconf;
    string homedir = getenv("HOME");
    LDef udef;
	if (!dir_exists(homedir + LINEAKDIR))
		create_homedir();
		
  	if (usrdeffilename == null) {
		usrdeffilename = homedir + LINEAKDIR;
		usrdeffilename += DEFFILE;
		if (file_exists(usrdeffilename)) {
	  		/* it exists, parse the contents */
	 		DefLoader defldr(usrdeffilename);
	  		udef = defldr.loadDef();
	  		if ( udef.isEmpty() )
	    		/* error parsing file */
	    		cerr << "*** Error occurred while reading definition data from " << usrdeffilename << endl;
		}
  	}
   	if (deffilename == null ) {
  		/* SET full path of definition files ($sysconfdir/lineakkb.def) */
    	deffilename = CONFDIR;
     	deffilename += "/";
      	deffilename += DEFFILE;
	}
	
	if ( deffilename != usrdeffilename ) {
	  	/* does the file exist? */
  		if (file_exists(deffilename)) {
			/* it exists, parse the contents */
			DefLoader sysdefldr(deffilename);
			def = sysdefldr.loadDef();
    		if ( def.isEmpty() ) {
	  			/* error parsing file */
	  			cerr << "*** Error occurred while reading definition data from " << deffilename << endl;
			}
  		}
    }
    	
   /* If we cannot load any keyboard definitions */
   if ( def.isEmpty()  && udef.isEmpty() ) {
		/* file does not exist */
    	cerr << "*** FATAL ERROR: No keyboard defintions could be loaded!" << endl;
		cleanexit(false);
  	}
	if (!udef.isEmpty())   {
	  	/** Append the user definition data to the system data */
		def.addKeyboards(udef);
		}
}

void lineak_core_functions::showkeyboards(LDef & def) {
	LKbdtable &tmp = def.getTable();
    
	cout << "\nLinEAK v" << VERSION << " -- supported keyboards:\n" << endl;
	cout << " [TYPE]\t\t[Full name]\n" << endl;
    for (LKbdIter m = tmp.begin(); m != tmp.end(); m++) {
        if (m->first != null)
    	   printf(" %s%s%s %s\n",(m->first).c_str(),strlen((m->first).c_str())>=7?"\t":"\t\t",strcasecmp(strip((m->second)->brand, "\"").c_str(),"other")==0?"":strip((m->second)->brand, "\"").c_str(),strip((m->second)->model, "\"").c_str());
    }
}
void lineak_core_functions::create_homedir(void) {
	string picsdir;
	string homedir = getenv("HOME");
	/** Make sure that the ~/.lineak directory exists */
	string lineakdir = homedir + LINEAKDIR;
	if (!dir_exists(lineakdir)) {
		if (mkdir(lineakdir.c_str(), 0755) == -1) {
        	cout << "*** FATAL ERROR: unable to create directory" << lineakdir << endl;
        	cleanexit(1);
    	}
  	}
   picsdir = homedir + PICSDIR;
	if (!dir_exists(picsdir)) {
		if (mkdir(picsdir.c_str(), 0755) == -1) {
        	cout << "*** ERROR: unable to create directory" << picsdir << endl;
    	}
  	}
}

/** We assume at this point that the global definitions objects are created */
void lineak_core_functions::create_new_conf(CmdPrefs & prefs, LDef & def) {
	string conffilename = prefs.getConffile();
	string cdromdev = prefs.getCdrom();
	string mixdev = prefs.getMixer();
	string homedir = getenv("HOME");
	string kbtype = prefs.getKbtype();
    LCommand blank;

	LConfig config;

	// Make sure we have definitions loaded and the kbtype is not empty.
	if (def.isEmpty() || kbtype == null) {
		cerr << "Attempting to create a config file with no keyboard definitions loaded" << endl;
		exit(false);
	}
	// Attempt to get the definition for the keyboard.
	if (!def.hasKeyboard(kbtype)) {
		cerr << "*** ERROR: Invalid keyboard type: " << kbtype<< " \nTo find out the supported keyboard types, use: 'lineakd -l'\n" << endl;
		exit(false);
	}
    create_homedir();
	/* set (globally) full path of config file ($HOME/.lineak/CONFIGFILE) */
  	if (conffilename == null )
    	conffilename = homedir + CONFFILE;
     
    /* no CD-ROM device specified? */
    if (cdromdev == null) {
		cout << "*** No CD-ROM device specified, using the default /dev/cdrom" << endl;
		cdromdev = DEFAULT_CDROM_DEVICE;
	}
    /* no mixer device specified? */
    if (mixdev == null) {
		cout << "*** No mixer device specified, using the default /dev/mixer" << endl;
		mixdev = DEFAULT_MIXER_DEVICE;
	}
    
    config = LConfig (kbtype,cdromdev, mixdev);
    config.setFilename( conffilename);
    LKbd kbd = def.getKeyboard(kbtype);
   	const SKeyTable & keytable = kbd.getKeys();
   	for (SKeyTableIter m = keytable.begin(); m != keytable.end(); m++)
		config.addKey(m->first,blank);

	Saver tmp(config.getFilename());

	if (!tmp.saveFile(config))
		exit(false);
	else {
		cout << "\n*** Creating fresh configuration in " << config.getFilename() << "\n      for keyboard type: " << kbtype << endl;
		cout << "NOTE: Now please edit the file and bind commands to the keys,\n      or use klineakconfig :)\n" << endl;
  }
}
/* show usage information on stdout */
void lineak_core_functions::showusage(int argc, char *argv[]) {
	cout << "\nLinEAK v" << VERSION << " -- Linux support for Easy Access and Internet Keyboards";
	cout << "\n";
	printf("Usage: %s [options...]\n", argv[0]);
	cout << "Valid options are:\n";
#ifdef HAVE_GETOPT_LONG
	cout << "    -l,        --kbd-list     	    Show all supported keyboard types\n";
	cout << "    -f FILE,   --conffile=FILE      Specify the .conf file\n";
	cout << "    -e FILE,   --deffile=FILE       Specify the .def file\n";
	cout << "    -d DEVICE, --cdrom-dev=DEVICE   Specify the CDROM device\n";
	cout << "    -m DEVICE, --mixer-dev=DEVICE   Specify the mixer device\n";
	cout << "    -c TYPE,   --config=TYPE        Create a new lineakd.conf file for keyboard TYPE\n";
	cout << "                                      (warning: old one will be overwritten!)\n";
//	cout << "    -b,        --background         Run in background\n";
	cout << "    -v,        --verbose            Print verbose messages\n";
	cout << "                                      (only visible when not running in background)\n";
	cout << "    -h,        --help               Print this message\n";
	cout << "    -r,        --reload             Reload the configuration files.\n";
	cout << "    -x,        --exit               Tell the daemon to exit.\n";
    #ifdef HAVE_XOSD
	cout << "    -n,        --xosd-font          Set the font the on-screen display will use. It must be in X format.\n";
    cout << "                                       i.e. -bitstream-charter-black-r-normal-*-*-240-*-*-p-*-iso8859-1\n";
	cout << "    -o,        --xosd-color         Set the color of the on-screen display. It must be given as a color code.\n";
    cout << "                                       i.e. #0aff00\n";
	cout << "    -t,        --xosd-timeout       Set the amount of time that the on-screen display keeps it's text on screen.\n";
	cout << "    -p,        --xosd-pos           Set the vertical position where xosd will display.\n";
    cout << "                                       Valid values are: top, middle, bottom\n";
	cout << "    -a,        --xosd-align         Set the horizontal alignment for the on-screen display.\n";
    cout << "                                       Valid values are: left, center, right\n";
	cout << "    -y,        --xosd-hoffset       Set the horizontal offset from the alignment (see above).\n";
	cout << "    -w,        --xosd-voffset       Set the vertical offset from the position (see xosd-pos above).\n";
	cout << "    -s,        --xosd-soffset       Set the offset for the shadow behind displayed text.\n";
    #endif
#else
	cout << "    -l            Show all supported keyboard types\n";
	cout << "    -f FILE       Specify the .conf file\n";
	cout << "    -e FILE       Specify the .def file\n";
	cout << "    -d DEVICE     Specify the CDROM device\n";
	cout << "    -m DEVICE     Specify the mixer device\n";
	cout << "    -c TYPE       Create a new lineakd.conf file for TYPE keyboard\n";
	cout << "                    (warning: old one will be overwritten!)\n";
	cout << "    -v,           Print verbose messages\n";
	cout << "                    (only visible when not running in background)\n";
	cout << "    -h            Print this message\n";
	cout << "    -r,           Reload the configuration files.\n";
	cout << "    -x,           Tell the daemon to exit.\n";
    #ifdef HAVE_XOSD
	cout << "    -n,           Set the font the on-screen display will use. It must be in X format.\n";
    cout << "                    i.e. -bitstream-charter-black-r-normal-*-*-240-*-*-p-*-iso8859-1\n";
	cout << "    -o,           Set the color of the on-screen display. It must be given as a color code.\n";
    cout << "                    i.e. 0aff00\n";
	cout << "    -t,           Set the amount of time that the on-screen display keeps it's text on screen.\n";
	cout << "    -p,           Set the vertical position where xosd will display.\n";
    cout << "                    Valid values are: top, middle, bottom\n";
	cout << "    -a,           Set the horizontal alignment for the on-screen display.\n";
    cout << "                    Valid values are: left, center, right\n";
	cout << "    -y,           Set the horizontal offset from the alignment (see above).\n";
	cout << "    -w,           Set the vertical offset from the position (see xosd-pos above).\n";
	cout << "    -s,           Set the offset for the shadow behind displayed text.\n";
    #endif
#endif
  	cout << "\nNOTE: if '-f', '-e', '-d', or '-m' parameters are used in conjunction with '-c', please\n      make sure you specify those BEFORE '-c' !\n";
  cout << endl;
}

/* these functions provide for a somewhat cleaner exit */
void lineak_core_functions::cleanexit (int status) {
//cout << "In cleanexit\n";
	#ifdef HAVE_XOSD
  	if (myXosd != 0) {
    	myXosd->cleanup();
    	delete (myXosd);
  	}
  	#endif
  	if (X != 0)
    	delete (X);
    
    msgPasser myMsg;
  	myMsg.quit();
  	lockCtrl lock;
  	lock.unlock();
  	exit(status);
}
void lineak_core_functions::signalexit(int sig_num) {
  do_exit();
}
void lineak_core_functions::signalrestart(int sig_num) {
  restart();
}
/* this is our "clean-up-that-mess" signal handler ;-) */
void lineak_core_functions::do_exit() {
	sigset_t mask_set; /* our signal masking set. */
  	sigset_t old_set;  /* the old mask set.   */
   //static int count = 0;
   
    #ifdef HAVE_XOSD
    static bool xosdcleanup = false;
    if(!xosdcleanup) {
      xosdcleanup = true;
  		  if (cmdprefs.getVerbose()) cout << "Cleaning up onscreen display" << endl;
    		  if (myXosd != 0) {
          myXosd->cleanup();
          delete (myXosd);
          myXosd = 0;
        }
    }
  	#endif

   	if (exec != 0) {
		  if (cmdprefs.getVerbose()) cout << "Cleaning up command displatcher and macro engine." << endl;
    	delete (exec);
     	exec = 0;
    }
    cout << "doneexit\n";
  	/** This is a critical section that we only want called once. */
  /* Put a semaphore here so that we can set doneexit atomically */
  sem_wait(&x_sem);
  	if (!doneexit) {
    	doneexit = true;

    /* mask (block) any further signals while we're inside this signal handler. */
  		sigfillset(&mask_set);
  		sigprocmask(SIG_SETMASK, &mask_set, &old_set);
    	sem_post(&x_sem);
  		/* now do some cleaning... */
  		if (cmdprefs.getVerbose()) cout << "Cleaning up xkb mappings" << endl;
    if (X != 0) {
  		  X->cleanup(myKbd);
          sleep(3);
  		  X->cleanup(myKbd);
      delete(X);
      X = 0;
    }

    	/* .. restore the old signal mask .. */
  		sigprocmask(SIG_SETMASK, &old_set, NULL);

    /* and finally exit.. */
  		//if (cmdprefs.getVerbose()) cout << "Cancelling messaging thread" << endl;
     // Must be called before we cancel the thread, or else the thread will be blocked
     // when we try to cancel it, and it will hang.
//    	sem_post(&x_sem);
    /* Stop the messagin thread */
    	//int ret = pthread_cancel(m_thread);
    	//if (ret != 0) {
    	//	perror("Thread cancellation failed");
    	//}

     /* Remove the message queue */
  		if (cmdprefs.getVerbose()) cout << "Removing message queue" << endl;
		msgPasser myMsg;
    	myMsg.quit();

    /** Destroy the semaphore */
    sem_destroy(&x_sem);

    /** Remove the lock */
  		if (cmdprefs.getVerbose()) cout << "Removing lock" << endl;
    lockCtrl lock;
    lock.unlock();
    exit(1);
  }
//  sleep(2);
  exit(1);
}

/* Reload .conf .def files and re-init program */
void lineak_core_functions::restart() {
	static bool donehup = false;
  	sigset_t mask_set; /* our signal masking set. */
  	sigset_t old_set;  /* the old mask set.   */
  /* Poor mans mutex */
  //sem_wait(&hup_sem);
  if (!donehup) {
		donehup = true;
    	//sem_post(&x_sem);
		/* mask (block) any further signals while we're inside this signal handler. */
  		sigfillset(&mask_set);
  		sigprocmask(SIG_SETMASK, &mask_set, &old_set);
   		bool verbose = cmdprefs.getVerbose();

   		if (verbose) cout << "Recieved a message to load a new configuration" << endl;
    
  		#ifdef HAVE_XOSD
  		if (verbose) cout << "*** Shutting down xosd safely..." << endl;
  		myXosd->cleanup();
  		#endif

  		/* reload .conf and .def file */
  		if (verbose) cout << "*** reloading config file " << myConfig.getFilename() << endl;
  		parseconffile(cmdprefs, myConfig);
  		if (verbose) cout << "*** reloading definition file " << myDef.getFilename() << endl;
  		parsedeffile(cmdprefs, myDef);

  		/* re-init the EAKeyboard */
  		if (verbose) cout << "*** re-initializing the keyboard" << endl;
    	myKbd = myDef.getKeyboard(myConfig.kbdtype);
			/** Update our keyboard with our configured commands */
			myKbd.setCommands(myConfig);
  		/* init the cdrom (disable auto eject) */
//			cdromCtrl myCdrom(myConfig.cdrom);
//  		/* Set the mixer device */
//  		soundCtrl mySound(myConfig.mixer);

  		/* re-init X/Xkb with our EAK keycodes/keysyms */
  		if (verbose) cout << "*** reconfiguring Xkb keymap with new keycodes/keysyms" << endl;
//  		X.cleanup(myKbd);
			X->initialize(myKbd);
	
  		/* .. restore the old signal mask .. */
  		if (verbose) cout << "*** rehash done" << endl;
  		sigprocmask(SIG_SETMASK, &old_set, NULL);
   
  		#ifdef HAVE_XOSD
  		if (verbose) cout << "*** Restarting xosd..." << endl;
  		myXosd->init(myConfig);
    	#endif
     
    	if (exec != 0) {
     		delete(exec);
  		#ifdef HAVE_XOSD
       	exec = new CommandExec(myKbd, myConfig, *myXosd, verbose);
			#else
  		  exec = new CommandExec(myKbd, myConfig, verbose);
			#endif
			}
			
    	donehup=false;
  }

  	return;
}

/**********************************************************************************************/

void lineak_core_functions::watch_messages() {
	/** Create a new thread */
	int res;
    pthread_attr_t thread_attr;
    res = pthread_attr_init(&thread_attr);

    res = sem_init(&x_sem, 0,1);
    if (res != 0) {
    	perror("Semaphore initialization failed");
     	exit(EXIT_FAILURE);
    }

	if (res != 0) {
		perror("Attribute creation failed for message handling thread");
		exit(EXIT_FAILURE);
	}
	res = pthread_attr_setdetachstate(&thread_attr, PTHREAD_CREATE_DETACHED);
	if (res != 0) {
		perror("Setting detached attribute failed for message handling thread");
		exit(EXIT_FAILURE);
	}
	res = pthread_create(&m_thread, NULL, handle_messages, NULL);
	if (res != 0) {
		perror("Thread creation failed for message handling");
		exit(EXIT_FAILURE);
	}
	(void)pthread_attr_destroy(&thread_attr);
}
void *lineak_core_functions::handle_messages(void *arg) {
	/** Handle messages */
	msgPasser myMsg;
	eak_msg msg;
	bool verbose = cmdprefs.getVerbose();

   	int res = pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,NULL);
	if (res != 0) {
		perror("Thread cancel state failed");
		exit(EXIT_FAILURE);
	}
	res = pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED,NULL);
//	res = pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,NULL);
	if (res != 0) {
		perror("Thread cancel type failed");
		exit(EXIT_FAILURE);
	}
		
	while (true) {
     	msg = myMsg.getMessage();
      if (msg.msg_type == EXIT) {
        	if (verbose) cout << "Exit message" << endl;
           // if (verbose) cout << "Message text = " << msg.directive.args << endl;
      		raise(SIGTERM);
      	}
      	if(msg.msg_type == HUP) {
        	if (verbose) cout << "Restart message" << endl;
            //if (verbose) cout << "Message text = " << msg.directive.args << endl;
         	//raise(SIGHUP);
          restart();
       }
      	if(msg.msg_type == TERM) {
       		pthread_exit(NULL);
       }
       msg.msg_type = 0;
       strcpy(msg.directive.args,"");
       sleep(1);
	}
  pthread_exit(NULL);
}
/* check if there's already a running lineakd for this $USER */
bool lineak_core_functions::is_running(string process) {
    uid_t myuid;
    pid_t mypid, pid;
    DIR *dir;
    struct dirent *d;
    struct stat s;
    string str, path;

    mypid = getpid();
    myuid = getuid();
   
   if ((dir = opendir("/proc")) == NULL)
   {
      cout << "Cannot open /proc" << endl;
      return -1;
   }
         
   while ((d = readdir(dir)) != NULL)
   {
      // See if this is a process
      if ((pid = atoi(d->d_name)) == 0)
         continue;
     
      // Ignore the pid of the current running process
      if (pid == mypid)
         continue;
  
      path = "/proc/" + string(d->d_name) + "/stat";
	    
      // Only processes with the same UID as the current running process
      if (stat(path.c_str(), &s) != 0 && s.st_uid != myuid)
         continue;
        
      // Open the status file and verify the program name
      ifstream in(path.c_str());
      if (in.is_open())
      {
         in >> str;
         in >> str;
         in.close();
         if (str.find("(" + process+")") != string::npos)
	 {
	    cout << "*** " << process << " is running (pid " << pid << ")" << endl;
            return true;
	 }
      }
   } 
    return false;
}
