/***************************************************************************
                          lcommand.cpp  -  description
                             -------------------
    begin                : ?  8? 20 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lcommand.h"
#include "functions.h"
#include <iostream>
using namespace std;

LCommand::LCommand() {
  isempty = true;
  ismacro = false;
  separator = ',';
  command = null;
  macro_type = null;
  args.clear();
  init_macros();
}

LCommand::LCommand(string icommand) {
   isempty = false;
   ismacro = false;
   separator = ',';
   macro_type = null;
   args.clear();
   command = icommand;
//   command = lineak_util_functions::strip_quotes(icommand);
	 init_macros();
   // Now, see if this thing is a macro, and if so, get the arguments.
   parse();
}

/* Copy constructor */
LCommand::LCommand(const LCommand& rhs) {
 if (this != &rhs ) {
	 init_macros();
   isempty = rhs.isempty;
   ismacro = rhs.ismacro;
   separator = rhs.separator;
   macro_type = rhs.macro_type;
   command = rhs.command;
   args.clear();
   for (vector<string>::const_iterator it = rhs.args.begin(); it != rhs.args.end(); it++ )
   	args.push_back(*it);
  }
}
void LCommand::init_macros() {
   specials.push_back(EAK_EJECT);
   specials.push_back(EAK_VOLUP);
   specials.push_back(EAK_VOLDOWN);
   specials.push_back(EAK_MUTE);
   specials.push_back(EAK_SLEEP);
   specials.push_back(EAK_SCREEN_LOCK);
}
LCommand &LCommand::operator=(const LCommand& rhs) {
 if (this != &rhs ) {
   isempty = rhs.isempty;
   ismacro = rhs.ismacro;
   separator = rhs.separator;
   macro_type = rhs.macro_type;
   command = rhs.command;
   args.clear();
   for (vector<string>::const_iterator it = rhs.args.begin(); it != rhs.args.end(); it++ )
   	args.push_back(*it);
  }
  return *this;
}

void LCommand::parse() {
  if (command != null) {
  	command = lineak_util_functions::strip(command, "\"#");
    macro_type = getMacroType();
    isempty = false;
    if (macro_type != null)
    {
      getMacroArgs();
    }
  }
}
LCommand::~LCommand(){
}
//void LCommand::remArg(string arg) {
//
//  VecIter it = args.find(arg);
//  args.erase(it);
//}

/* You should not need to call this. The default argument separator is a comma. It is set in both constructors to the default.
    If however, you do change the separator, and (provided that command is not null), you need to call parse() */
void LCommand::setSeparator(string sep) {
    separator = sep;
    if (command != null)
      parse();
    else {
      isempty = false;
      ismacro = false;
      macro_type = null;
      args.clear();
    }
}

/** Determines if the command is a special EAK command or not. */
bool LCommand::isMacro(){
    /** go through our list of special commands and if the command contains those characters
          return true */
    for ( vector<string>::iterator it=specials.begin(); it < specials.end(); it++) {
      if ( command.find(*it) == 0) {
        return true;
      }
    }
    return false;
}
/** Determine the type of Macro this command is and return */
string LCommand::getMacroType() {
    for ( vector<string>::iterator it=specials.begin(); it < specials.end(); it++) {
        if (command.find(*it) == 0) {
          ismacro = true;
          return (*it);
        }
    }
    ismacro = false;
    return null;
}
/** Later we want to do this on config file load. Parse in the loader and extend the LConfig class so that a command
    is not just a string, but a class that contains the command, and a vector<string> of arguments */
bool LCommand::getMacroArgs() {
    int b,e;
    
    // Strip off the '(' and ')' characters. tmp contains the argument string.
    b = command.find('(');
    // If we don't find an opening brace, we have no arguments
    if (b <= 0)
      return false;

    // So... we've found something to parse.
    string tmp = command.substr(b+1, command.length()-1);
    e = tmp.find(')');
    tmp = tmp.substr(0,e);
    // If we actually have no arguments, return null.
    if ( tmp == "") {
//      args.clear();
      return false;
    }

    /** We have arguments now.
          Traverse the remaining string, and rip out the arguments. The string should look like this:
          "/dev/cdrom,/devcdrom1"  or
          "5, /dev/mixer"  or
          "-1"
    */
    do {
      e = tmp.find(separator);
      if (e <0) {
        args.push_back(tmp);
        break;
      } else {
        args.push_back(tmp.substr(0,e));
        tmp = tmp.substr(e+1,tmp.length());
      }
    } while (tmp != "" );

    return true;
}

#ifdef _DEBUG_
void LCommand::print(ostream &out) {
  out << "command = " << command << endl;
  out << "separator = " << separator << endl;
  out << "isempty = " << isempty << endl;
  out << "ismacro = " << ismacro << endl;
  if (ismacro) {
    out << "   macro_type = " << macro_type << endl;
    out << "   There are: " << args.size() << " macro arguments: { ";
	for (vector<string>::const_iterator it = args.begin(); it != args.end(); it++)
		out << *it << " ";
	out << "} " << endl;
  }

}
#endif

ostream & operator<<(ostream &out, LCommand &rhs) {
  out << rhs.getCommand() << endl;
  return out;
}
ostream & operator<<(ostream &out, const LCommand &rhs) {
  out << const_cast<LCommand &>(rhs).getCommand() << endl;
  return out;
}
