/***************************************************************************
                          lconfig.cpp  -  description
                             -------------------
    begin                : Fri Jan 17 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lconfig.h"
#include "functions.h"

using namespace lineak_util_functions;

LConfig::LConfig() {
	kbdtype = null;
	cdrom = DEFAULT_CDROM_DEVICE;
	mixer = DEFAULT_MIXER_DEVICE;
	conffilename = null;
    #ifdef HAVE_XOSD
     /** Set the XOSD values to lineakd defaults */
    XOSD_font = XOSD_FONT;
    XOSD_color = XOSD_COLOR;
    XOSD_timeout = XOSD_TIMEOUT;
    XOSD_pos = XOSD_POS;
    XOSD_align = XOSD_ALIGN;
    XOSD_hoffset = XOSD_HOFFSET;
    XOSD_voffset = XOSD_VOFFSET;
    XOSD_soffset = XOSD_SOFFSET;
    #endif
}
//LConfig::LConfig(ConfigLoader &ldr) {
//	*this = ldr;
//	//LConfig conf = ldr.loadConfig();
//	//kbdtype = conf.kbdtype;
//	//cdrom = conf.cdrom;
//	//mixer = conf.mixer;
//	//conffilename = conf.getFilename();
//	//comtable
//}
LConfig::LConfig(const LConfig & rhs) {
	if (this != &rhs) {
		comtable.clear();
		kbdtype = rhs.kbdtype;
		cdrom = rhs.cdrom;
		mixer = rhs.mixer;
		conffilename = rhs.conffilename;
		comtable = rhs.comtable;
        #ifdef HAVE_XOSD
        XOSD_font = rhs.XOSD_font;
        XOSD_color = rhs.XOSD_color;
        XOSD_timeout = rhs.XOSD_timeout;
        XOSD_pos = rhs.XOSD_pos;
        XOSD_align = rhs.XOSD_align;
        XOSD_hoffset = rhs.XOSD_hoffset;
        XOSD_voffset = rhs.XOSD_voffset;
        XOSD_soffset = rhs.XOSD_soffset;
        #endif
	}
}
LConfig & LConfig::operator=(const LConfig & rhs) {
	if (this != &rhs) {
		comtable.clear();
		kbdtype = rhs.kbdtype;
		cdrom = rhs.cdrom;
		mixer = rhs.mixer;
		conffilename = rhs.conffilename;
		comtable = rhs.comtable;
    #ifdef HAVE_XOSD
    XOSD_font = rhs.XOSD_font;
    XOSD_color = rhs.XOSD_color;
    XOSD_timeout = rhs.XOSD_timeout;
    XOSD_pos = rhs.XOSD_pos;
    XOSD_align = rhs.XOSD_align;
    XOSD_hoffset = rhs.XOSD_hoffset;
    XOSD_voffset = rhs.XOSD_voffset;
    XOSD_soffset = rhs.XOSD_soffset;
    #endif
    
	}
	return *this;
}
LConfig::LConfig(ConfigDirectives &directives) {
	cdrom = directives["CdromDevice"];
	kbdtype = directives["KeyboardType"];
	mixer = directives["MixerDevice"];
	screensaver = directives["Screensaver"];
	
	if (cdrom == null)
		cdrom = DEFAULT_CDROM_DEVICE;
	if (mixer == null)
		mixer = DEFAULT_MIXER_DEVICE;

    #ifdef HAVE_XOSD
    XOSD_font = directives["XOSD_font"];
    XOSD_color = directives["XOSD_color"];
    string position = directives["XOSD_pos"];
              if (position == "top")
                XOSD_pos = XOSD_top;
              if (position == "middle")
                XOSD_pos = XOSD_middle;
              if (position == "bottom" || position == "")
                XOSD_pos = XOSD_bottom;
    string align = directives["XOSD_align"];
              if (align == "left")
                XOSD_align = XOSD_left;
              if (align == "center" || align == "")
                XOSD_align = XOSD_center;
              if (align == "right")
                XOSD_align = XOSD_right;

    XOSD_timeout = atoi((directives["XOSD_timeout"]).c_str());
    XOSD_hoffset = atoi((directives["XOSD_hoffset"]).c_str());
    XOSD_voffset = atoi((directives["XOSD_voffset"]).c_str());
    XOSD_soffset = atoi((directives["XOSD_soffset"]).c_str());
    #endif

}

LConfig::LConfig(string ikbdtype, string icdrom, string imixer) : kbdtype(ikbdtype), cdrom(icdrom),mixer(imixer)  {
	if (cdrom == null)
		cdrom = DEFAULT_CDROM_DEVICE;
	if (mixer == null)
		mixer = DEFAULT_MIXER_DEVICE;

    #ifdef HAVE_XOSD
     /** Set the XOSD values to lineakd defaults */
    XOSD_font = XOSD_FONT;
    XOSD_color = XOSD_COLOR;
    XOSD_timeout = XOSD_TIMEOUT;
    XOSD_pos = XOSD_POS;
    XOSD_align = XOSD_ALIGN;
    XOSD_hoffset = XOSD_HOFFSET;
    XOSD_voffset = XOSD_VOFFSET;
    XOSD_soffset = XOSD_SOFFSET;
    #endif

}
LConfig::~LConfig(){
}
/** Return a list of keynames */
const Vector& LConfig::getKeyNames(){
	vec.clear();
	for (HashIter i = comtable.begin(); i != comtable.end(); i++)
		vec.push_back(i->first);
	return vec;
}
/** Return a map of commands indexed by keycodes. */
IKeyTable LConfig::getCommandIndex(const SKeyTable& skeys) {
	IKeyTable table;
	string comm;
	for ( SKeyTableIter i = skeys.begin(); i != skeys.end(); i++ ) 
		table[(i->second).keycode] = comtable[i->first];
	return (table);
}
/** Return a map of commands indexed by keycodes. */
IKeyTable LConfig::getCommandIndex(const LKbd& keyboard) {
	return (getCommandIndex(keyboard.getKeys()));
}
LCommand& LConfig::getCommand(string key) {
	return comtable[key];
}
void LConfig::setCommand(string key, LCommand command) {
	comtable[key] = command;
}
void LConfig::addKey(string key, LCommand command, bool respect_toggle) {
//  cout << "Setting command\n";
//  lineak_util_functions::inspect_vector(command.getArgs());
	if (!respect_toggle)
		setCommand(key, command);
		
}
void LConfig::setFilename(string file) {
	conffilename = file;
}
bool LConfig::isEmpty(){
	return (comtable.empty() || (kbdtype == null && conffilename == null));
}

void LConfig::print(ostream &out) const {
      
    out << CONF_HEADER;
    out << "KeyboardType = " << kbdtype << endl;
   	out << "CdromDevice = " << cdrom << endl;
   	out << "MixerDevice = " << mixer << endl;
   	out << "Screensaver = " << screensaver << endl;

    #ifdef HAVE_XOSD
    string pos, align;
    // Save our enums back as strings.
    if (XOSD_pos == XOSD_top)
      pos = "top";
    if (XOSD_pos == XOSD_middle)
      pos = "middle";
    if (XOSD_pos == XOSD_bottom)
      pos = "bottom";
    if (XOSD_align == XOSD_left)
      align = "left";
    if (XOSD_align == XOSD_center)
      align = "center";
    if (XOSD_align == XOSD_right)
      align = "right";

    out << "XOSD_font = " << XOSD_font << endl;
    out << "XOSD_color = " << XOSD_color  << endl;
    out << "XOSD_timeout = " << XOSD_timeout << endl;
    
    out << "XOSD_pos = " << pos << endl;
    out << "XOSD_align = " << align << endl;
    
    out << "XOSD_hoffset = " << XOSD_hoffset << endl;
    out << "XOSD_voffset = " << XOSD_voffset << endl;
    out << "XOSD_soffset = " << XOSD_soffset << endl;
    #endif

    out << endl;
    string command;
    LCommand comm;
		for (HashIter i = comtable.begin(); i != comtable.end(); i++ ) {
        comm = i->second;
        command = comm.getCommand();
        if (command == null)
          command = "";
		out << i->first << " = " << command << endl;
    }
}
