/***************************************************************************
                          lconfig.h  -  description
                             -------------------
    begin                : Fri Jan 17 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LCONFIG_H
#define LCONFIG_H

#if __GNUC__ >= 3
   #include <fstream>
#else
   #include <fstream.h>
#endif

#include "../config.h"
#ifdef HAVE_XOSD
  #include "xosdctrl.h"
#endif

#include <map>
#include <vector>
#include <iostream>
#include "definitions.h"
#include "configdirectives.h"
#include "lconfigdata.h"
#include "lkbd.h"
#include "lkey.h"
#include "lcommand.h"

typedef map<const string, LCommand> Hashtable;
typedef Hashtable::const_iterator HashIter;

typedef map<const int,  LCommand> IKeyTable;
typedef IKeyTable::const_iterator IKeyTableIter;

typedef map< string, LKey> SKeyTable;
typedef SKeyTable::const_iterator SKeyTableIter;

/**Lineak user config file.
  *@author Sheldon Lee Wen
  */
class LConfig : public LConfigData {
public: 
	LConfig();
	LConfig(const LConfig &rhs);
	LConfig(ConfigDirectives &directives);
	LConfig(string ikbdtype, string icdrom, string imixer);
	LConfig & operator=(const LConfig &rhs);
	virtual ~LConfig();
  /** Change this class to contain these thi9ngs in a map. Then overload the [] and []= operators so that we can
  		get these values by doing string cdrom = myConfig["CdromDevice"]; or myConfig["CdromDevice"] = cdrom;
    	This makes it easier for me to add configuration directives to the config file. This way I don't have to
     	extend the class for every directive we understand */
      /** private:
      		map<string,string> configs;
        	public:
         	string& operator[](string key) { return(configs[key]); }
          string& operator[]=(string key, const string &value) { configs[key] = value; }
          or something like this....
      */
	string kbdtype;
	string cdrom;
	string mixer;
	string screensaver;

    #ifdef HAVE_XOSD
    /** XOSD Settings */
    string XOSD_font;
    string XOSD_color;
    int XOSD_timeout;
    xosd_pos XOSD_pos;
    xosd_align XOSD_align;
    int XOSD_hoffset;
    int XOSD_voffset;
    int XOSD_soffset;
    #endif
   	
  	/** Return a list of keynames */
  const Vector& getKeyNames();
  	/** Return a map of commands indexed by keycodes. */
  	IKeyTable getCommandIndex(const SKeyTable& skeys) ;
  	/** Return a map of commands indexed by keycodes. */
  	IKeyTable getCommandIndex(const LKbd& keyboard) ;
   /** Return a command string given a key code */
	LCommand& getCommand(string key);
	/** Set the command for a key */
	void setCommand(string key, LCommand command);
	/** Add a key and command mapping. */
	void addKey(string key, LCommand command, bool respect_toggle = false);
	inline string getFilename() const { return conffilename; }
	void setFilename(string file);
    bool isEmpty();
	/** print */
	void print(ostream&out) const;
	//void print(void) const;
private:
	Hashtable  comtable;
//    map<const string, LCommand> comtable;
	string conffilename;
	Vector vec;
};

#endif
