/***************************************************************************
                          lkbd.h  -  description
                             -------------------
    begin                : Fri Jan 17 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LKBD_H
#define LKBD_H

#include <string>
#include <vector>
#include <map>
#include <queue>

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/XKBlib.h>
#include <X11/extensions/XKBfile.h>

#include "definitions.h"
#include "lconfigdata.h"

/**Lineak Keyboard
  *@author Sheldon Lee Wen
  */

class LKbd;
class LKey;
class LConfig;
class LCommand;

// Map of keyboards indexed by config name.  
//typedef map<string,LKbd*> LKbdtable;
//typedef LKbdtable::const_iterator LKbdIter;
// Map of keynames and their respective keycodes.
typedef map<string, LKey> SKeyTable;
typedef SKeyTable::const_iterator SKeyTableIter;

class LKbd : public LConfigData {
public: 
	LKbd();
	LKbd(string iname, string ibrand, string imodel);
	virtual ~LKbd();
	
	string name;
	string brand;
	string model;
	
    void addKey(const string name, int keycode);
    void setCommands(LConfig &config);
    void setCommand(string name, LCommand command);
  	bool operator==(const LKbd &rhs);
  	/** Return a vector of all of the keycodes. */
  	vector<int> getKeyCodes();
  	/** Return a vector of all of the key names. */
    Vector getKeyNames();
    /** Get a table of key to keycode mappings, indexed by stringname. */
    const SKeyTable& getKeys() const;
    /** Get a reference to a key, indexed by keyname or keycode */
    LKey& getKey(string name);
    LKey& getKey(int keycode);
    
    /** Is the object empty? */
    bool isEmpty();
    /** Print */
	void print(ostream&out) const;
	//void print(void) const;

private:
	SKeyTable keytable;
    // We only allow the definition loader to create an empty keyboard.
//	friend class DefLoader;
//	friend class LDef;

};



//ostream & operator<<(ostream &out, LKbd &rhs);
#endif
