/***************************************************************************
                          lkey.h  -  description
                             -------------------
    begin                : Fri Feb 28 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LKEY_H
#define LKEY_H

#include <string>
#include <vector>
#include <map>
#include <queue>

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/XKBlib.h>
#include <X11/extensions/XKBfile.h>

#include "definitions.h"
#include "lconfigdata.h"
#include "lcommand.h"

/**This class contains a key.
  *@author Sheldon Lee Wen
  */
class LKey {
public:
	LKey(); 
	LKey(string name, int code, LCommand com);
	~LKey();

  	string getKeyname() { return keyname; }
    void setKeyname(const string & name);
    const Vector & getKeynames() { return keynames; }
    string getDisplayName();
    LCommand getCommand();
    void setCommand(string name, LCommand command);
    void setCommand(LCommand command);
		KeySym getKeySym() { return keysym; }
		void clearKeycount() { keycount = 0; }
		void print(ostream&out);
    /** Return whether or not we are using this key as a toggleable key */
    bool isToggle() const;
    /** Return whether or not this is a toggleable key. The distinction is
    		necessary for xosd where you have a config file entry of the form:
      	key|key = command
       	In this case the key is not being used as toggleable in the sense that
        each key press is bound to a different command. We are using it as
        a normal key. However, for the sake of the on screen display, we want
        to toggle the display. */
    bool isDisplayToggle() const { return dtoggle; }
    void setToggle(bool tog) { toggle = tog; }
    void setDisplayToggle(bool tog) { dtoggle = tog; }
    /** If isToggle() , rotate the key to the next display and command state. */
    void toggleKeyState();
    int keycode;

private:
		void init();
  	KeySym keysym;
   	static int keycount;
    bool toggle,dtoggle;
    string keyname;
   	Vector keynames;
    map<string,LCommand> commands;
    queue<string> display_names;

};

ostream & operator<<(ostream &out, LKey &rhs);

#endif
