/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Wed Jan  1 14:03:54 EST 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#include <string>
#include <iostream>

#if __GNUC__ >= 3
   #include <ostream>
#else
   	#include <ostream.h>
#endif

// lineakd.c definitions, not sure if they are necessary here
/* system header files */

extern "C" {
#include <signal.h>
#include <X11/Xlib.h>
#include <X11/Xlibint.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Xos.h>
#include <X11/Xmu/Error.h>
#include <X11/Xmu/Xmu.h>
#include <X11/extensions/scrnsaver.h>
}

#ifndef X_NOT_STDC_ENV
  #include <stdlib.h>
#else
  extern char *getenv();
#endif


#ifdef HAVE_XOSD
  #include "xosdctrl.h"
	xosdCtrl *myXosd = 0;
#endif

#include "eakprocs.h"
#include "functions.h"
#include "configloader.h"
#include "defloader.h"
#include "lkbd.h"
#include "ldef.h"
#include "saver.h"
#include "xmgr.h"
#include "cmdprefs.h"
#include "soundctrl.h"
#include "cdromctrl.h"
#include "msgpasser.h"
#include "lockctrl.h"
#include "commandexec.h"

using namespace lineak_util_functions;
using namespace lineak_core_functions;
using namespace std;

/** Lineakd global objects */
LKbd myKbd(null,null,null);
LConfig myConfig;
LDef myDef;
Xmgr *X = 0;
CmdPrefs cmdprefs;
CommandExec *exec = 0;

int main(int argc, char *argv[])
{
	/** Make sure that the ~/.lineak directory exists */
	string homedir = getenv("HOME");
	/** Determine whether or not we are enabled or disabled
			this is used for checking for the screen saver */
//    bool enabled = true;
  /** We should be able to do this stuff w/o having an X connection */
  /* parse command line arguments */
  if ( !cmdprefs.parseArgs(argc,argv) )
	  exit(1);

  if (cmdprefs.showUsage()) {
   	showusage(argc, argv);
   	exit(1);
  }
  if (cmdprefs.showKeyboards()) {
	  parsedeffile(cmdprefs, myDef);
    showkeyboards(myDef);
    exit(1);
  }
	if(cmdprefs.createNewConfig()) {
		/* read the configuration and keyboard definition file */
		parsedeffile(cmdprefs, myDef);
		create_new_conf(cmdprefs, myDef);
		exit(1);
	}
   // Start the things that require cleanexit.
  /** End of Displayless functions. */
  	/** Display required function from here on. */
  	if (!X) {
//   		cout << "Going to display required functions\n";
    	X = new(Xmgr);
//     	cout << "got X object\n";
     }
  	if (!X->getDisplay()) {
    	cerr << "Could not open the display. Exiting." << endl;
    	exit(false);
  	}
  	/** Create the users .lineak directory */
		create_homedir();

  	/* everything went according to plan thisfar... some signal-handlers for cleaner exiting */
  	signal(SIGTERM, signalexit);
  	signal(SIGABRT, signalexit);
  	signal(SIGINT, signalexit);
  	/* and one so we won't have to wait() for child processes ;) */
  	signal(SIGCLD, SIG_IGN);
  	/* and for a rehash when we catch SIGHUP */
  	signal(SIGHUP,signalrestart);
	/* read the configuration and keyboard definition file */
    parseconffile(cmdprefs, myConfig);
//	cout << "Parsing def\n";
	parsedeffile(cmdprefs, myDef);
//	cout << "Setting opts\n";
//    inspect_vector(myConfig.getCommand("VolumeUp").getArgs());

	/** Update the config object with any command line options */
	cmdprefs.setOpts(myConfig);
  /* build our EAKeyboard with the data from deffile and conffile */
	/** get the keyboard from our config file */
//	cout << "Getting my keyboard\n";
  	myKbd = myDef.getKeyboard(myConfig.kbdtype);

   	if (myKbd.isEmpty()) {
    	cout << "Could not load a keyboard definition for the configured keyboard " << myConfig.kbdtype << endl;
     	cleanexit(false);
    }
	/** Update our keyboard with our configured commands */
	myKbd.setCommands(myConfig);
  	/* init the cdrom (disable auto eject) */
//	cdromCtrl myCdrom(myConfig.cdrom);
  	/* Set the mixer device */
//  	soundCtrl mySound(myConfig.mixer);
   // Start the things that require cleanexit.

  /* check if a pid file exists and take care of it */
  // With xosd, it doesn't work anymore. Switch to a lockfile method instead.
  lockCtrl lock;
  if (lock.isLocked()) {
    #ifdef USE_LOCKFILE
  	// Attempt to find any lineakd processes running.
   	string process = "lineakd";
  	if (is_running(process)) {
    	cerr << "Failed to start, lineakd is already running." << endl;
    	cerr << "If this is not the case, try removing the file ~/.lineak/lineakd.lock" << endl;
    	exit(false);
    }
    #else
    	cerr << "Failed to start, lineakd is already running." << endl;
    	exit(false);
    #endif    
  }
  else
  	lock.lock();

 	msgPasser messages;
	watch_messages();
	/* init X/Xkb with our EAK keycodes/keysyms */
  XkbEvent xev;
	Display* display;
  if(cmdprefs.getVerbose())
      cout << "Initializing the display\n";
  X->initialize(myKbd);
  display = X->getDisplay();
  
  /* initialize XOSD system */
  #ifdef HAVE_XOSD
  myXosd = new xosdCtrl(myConfig);
//    myXosd->setScrollAge(true);
    //myXosd->setForcewait(true);
  if(cmdprefs.getVerbose())

  myXosd->init();
  // myXosd->setScrollAge(true);
  #endif

  /* alright, we're going to loop forever now. only signals can interrupt us. */
  #ifdef HAVE_XOSD
  exec = new CommandExec(myKbd, myConfig, *myXosd, cmdprefs.getVerbose());
  #else
  exec = new CommandExec(myKbd, myConfig, cmdprefs.getVerbose());
  #endif

  while(1) {
			/* wait for next event */
			XNextEvent(display, &xev.core);

			/* should we respond to this event? */
    		if ( xev.type == X->xkbEventCode()+XkbEventCode && xev.any.xkb_type == XkbActionMessage ) {
      			if (cmdprefs.getVerbose()) printf("Keycode %d pressed\n", xev.message.keycode);
		  		/* if we are using xosd, output our display here */
      			#ifdef HAVE_XOSD
				/* Should we age the display? */
                if (myXosd->enabled()) {
		  				myXosd->agenscroll();
                }
      			#endif
	  			/* find the corresponding key, and execute it's command */
                exec->exec(xev.message.keycode);
			}
			if ( xev.core.type == MappingNotify ) {
		  		if (!X->xkbRemapped())
			  		X->xkbRemapped(true); // =1;
		  		else {
			  		if (cmdprefs.getVerbose())
				  		cout << "Reclaiming keyboard map from MappingNotify event" << endl;
			  		X->xkbRemapped(false); // = 0;
			  		X->initialize(myKbd);
		  		}
			}
	
    #ifdef HAVE_XOSD
    if (myXosd->enabled()) {
	  	myXosd->age();
    }
    #endif
  }
  /* that's all folks.. */
  do_exit();
  return true;
}

