/**************************************************************************
                          xmgr.cpp  -  description
                             -------------------
    begin                : Tue Feb 18 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "xmgr.h"
#include "eakprocs.h"
#include "lkbd.h"
#include "cmdprefs.h"
#include "lkey.h"
#include <signal.h>


extern CmdPrefs cmdprefs;

using namespace lineak_core_functions;

int Xmgr::nrretries=0;
bool Xmgr::got_xkb=false;
bool Xmgr::caughtXerror = false;
bool Xmgr::xkbremapped = false;

Xmgr::Xmgr(string display) : displayname(display) {
	xkbOpcode = 0;
	EventCode = 0;
  	if ( !(ourdisplay = openDisplay(displayname, xkbOpcode, EventCode))) {
   	    cerr << "Could not open the display. Exiting\n";
    	cleanexit(1);
    }

  	XSetErrorHandler(&Xhandler);
	XSetAfterFunction(ourdisplay, &dummyAfterFunc);

	//if (verbosemode) cout << "Getting a new map" << endl;
  	/* get the xkb map, and back it up. */
  	xkb = XkbGetMap(ourdisplay, XkbAllMapComponentsMask, XkbUseCoreKbd);
  	if (!xkb) {
  	  cerr << "FATAL ERROR: XkbGetMap failed" << endl;
  	  do_exit();
  	}
}

Xmgr::~Xmgr(){
	if (xkb != NULL)
    	XkbFreeClientMap(xkb, XkbAllMapComponentsMask, True);
}
/* X error handler. When a BadValue error occurs during XkbSetMap, we want to retry */
/* Right now we are just ignoring errors */
int Xmgr::Xhandler(Display* theDisplay, XErrorEvent* xev) {
//    // BadValue Error   (these numbers might not be platform or version independant?!!!):
//    // xev->request_code = 150 (XKEYBOARD)
//    // xev->minor_code   = 9   (XkbSetMap)
//	  if (nrretries < MAXRETRIES) {
//		          nrretries++;
//                  cout << "nrretries = " << nrretries << endl;
//              if (xev->request_code==150 && xev->minor_code==9) {
//    	           caughtXerror = true;
//                  cerr << "*** Xlib error caught ***" << endl;
//              }
//  	} else {
//		/* max nr retries failed, quit to prevent race condition */
//    	        cerr << "*** Xlib error caught, " << MAXRETRIES << " retries failed" << endl;
//		        cerr << "*** We should exit here." << endl;
//                raise(SIGTERM);
//		        //cleanexit(1);
//  	}
  	return 0;
}
/** commit the keycode to the keyboard map */
void Xmgr::commitXKBChanges(int tcode) {
    XkbMapChangesRec    mapChangeRec;

    /* Commit the change back to the server */
    bzero(&mapChangeRec, sizeof(mapChangeRec));
    mapChangeRec.changed = XkbKeySymsMask | XkbKeyTypesMask;
    mapChangeRec.first_key_sym = tcode;
    mapChangeRec.num_key_syms = 1;
    //        mapChangeRec.first_key_act = tcode;
    //        mapChangeRec.num_key_acts = 1;
    mapChangeRec.first_type = 0;
    mapChangeRec.num_types = xkb->map->num_types;
    if ( ! XkbChangeMap(ourdisplay, xkb, &mapChangeRec) ) {
    	cerr << "FATAL ERROR: map changed failed" << endl;
	  	do_exit();
    }
}
/* Get a hold on the display */
Display* Xmgr::openDisplay(string &dpyName, int &opcodeRtrn, int &evBaseRtrn) {
  	int mjr,mnr,error;
  	char *TEMPdpyName = NULL;
  	Display *dpy;

  if (dpyName == null) {
       if (getenv("DISPLAY") != NULL) {
    	   dpyName = getenv("DISPLAY");
          TEMPdpyName = (char *)malloc((strlen(dpyName.c_str())+1)*sizeof(char));
          sprintf(TEMPdpyName, "%s",dpyName.c_str());
       }
       else {
          dpyName = null;
          TEMPdpyName = NULL;
       }
  }
  else {
      TEMPdpyName = (char *)malloc((strlen(dpyName.c_str())+1)*sizeof(char));
      sprintf(TEMPdpyName, "%s",dpyName.c_str());
  }
  
  	mjr = XkbMajorVersion;
  	mnr = XkbMinorVersion;
   dpy = XkbOpenDisplay(TEMPdpyName, &evBaseRtrn,NULL,&mjr,&mnr,&error);
  	if (dpy == NULL) {
    	switch (error) {
        	case XkbOD_BadLibraryVersion:
                cerr << "This program was compiled with XKB version " << XkbMajorVersion << "." << XkbMinorVersion;
                cerr << "     your X library supports incompatible version " << mjr << "." << mnr << endl;
                free(TEMPdpyName);
                return NULL;
                break;
            case XkbOD_ConnectionRefused:
                cerr << "Cannot open display " << dpyName << endl;
                free(TEMPdpyName);
                return NULL;
                break;
            case XkbOD_NonXkbServer:
                cerr << "XKB extension not present on " << dpyName << endl;
                free(TEMPdpyName);
                return NULL;
                break;
            case XkbOD_BadServerVersion:
                cerr << "This program was compiled with XKB version " << XkbMajorVersion << "." << XkbMinorVersion;
                cerr << "Server " << dpyName << " uses incompatible version " << mjr << "." << mnr << endl;
                free(TEMPdpyName);
                return NULL;
                break;
            default:
                cout << "Unknown error " << error << " from XkbOpenDisplay" << endl;
                free(TEMPdpyName);
                return NULL;
                break;
      	}
    } else {
      	if (opcodeRtrn)
        	XkbQueryExtension(dpy, &opcodeRtrn, &evBaseRtrn,NULL,&mjr,&mnr);
    }
    free(TEMPdpyName);
    return dpy;
}
/* initialize X (xkb) so it understands the extra keys */
void Xmgr::cleanup(LKbd& myKbd) {
	XkbMessageAction xma;
  	XkbMapChangesRec mapChangeRec;
	bool verbosemode = cmdprefs.getVerbose();
        if (ourdisplay != NULL) {
  	/* construct the message action struct */
  	xma.type = KeyPress;
  	xma.flags = XkbSA_MessageGenKeyEvent;
  	strcpy((char *)xma.message, " ");

//	if (xkb != NULL)
//    	XkbFreeClientMap(xkb, XkbAllMapComponentsMask, True);
//
// 	/* get the xkb map */
//  	xkb = XkbGetMap(ourdisplay, XkbAllMapComponentsMask, XkbUseCoreKbd);
//  	if (!xkb) {
//     	cerr << "FATAL ERROR: XkbGetMap failed" << endl;
//     	do_exit();
//   }
// /* Try restoring the keyboard map from the saved one */
//  Can't figure out how to make it work.
//   if (got_xkb) {
//   		if(!XkbSetMap(ourdisplay, XkbAllMapComponentsMask, xkb_backup)) {
//     		cerr << "FATAL ERROR: XkbSetMap failed. Could not restore the keymap" << endl;
//       		do_exit();
//         }
//	}

//    myKbd = sysDef.getKeyboard(myConfig.kbdtype);
	vector<int> keycodes = myKbd.getKeyCodes();

  	/* add our new keysyms to the key codes of our display */
   	for (int i=0; i != (int)keycodes.size(); ) {
		int keycode = keycodes[i];

    	/* change the keysym */
    	if ( XkbResizeKeySyms( xkb, keycode, 0 ) == NULL ) {
        	cout << "FATAL ERROR: resize keysym failed" << endl;
	  		do_exit();
    	}
    	/* clear the keysym from keycode */
    	commitXKBChanges(keycode);
    	commitXKBChanges(keycode);
    	/* assign the message action to the key code */
    	(&(xkb->server->acts[ xkb->server->key_acts[keycode] ]))->msg = xma;
    	/* commit the change back to the server. */
    	bzero(&mapChangeRec, sizeof(mapChangeRec));
    	mapChangeRec.changed = XkbKeyActionsMask;
    	mapChangeRec.first_key_act = keycode;
    	mapChangeRec.num_key_acts = 0;
    	if ( XkbChangeMap(ourdisplay, xkb, &mapChangeRec) ) {
      		if (verbosemode) cout << "Xkb map removed done for keycode " << keycode << endl;
    	} else {
      		cerr << "FATAL ERROR: map changed failed" << endl;
	  		do_exit();
    	}
        i++;
//		if (!caughtXerror)
//	  		i++;
//		else {
//	  		if (verbosemode) cout << "*** caught X error, retrying for keycode " << keycode << endl;
//	  		caughtXerror = false;
//		}
  	}

  /* Select the ActionMessage event under any circumstances */
/*  if ( !XkbSelectEvents(ourdisplay, XkbUseCoreKbd, XkbAllEventsMask, XkbAllEventsMask)) {
    printf("FATAL ERROR: Couldn't select desired XKB events\n");
    do_exit(1);
  } */
  }
}
/* initialize X (xkb) so it understands the extra keys */
void Xmgr::initialize(LKbd& myKbd) {
	XkbMessageAction xma;
  	XkbMapChangesRec mapChangeRec;
  	int types[1];
	bool verbosemode = cmdprefs.getVerbose();
   /* construct the message action struct */
  	xma.type = XkbSA_ActionMessage;
  	xma.flags = XkbSA_MessageOnPress;
  	strcpy((char *)xma.message, " ");
    vector<int> keycodes = myKbd.getKeyCodes();
  	/* add our new keysyms to the key codes of our display */
   	for (int i=0 ; i != (int)keycodes.size();) {
         int keycode = keycodes[i];

      	//if (verbosemode) cout << "Freeing old Xkb map" << endl;

//        if (xkb != NULL)
//        	XkbFreeClientMap(xkb, XkbAllMapComponentsMask, True);
//
//      	//if (verbosemode) cout << "Getting a new map" << endl;
//     	/* get the xkb map, and back it up. */
//    	xkb = XkbGetMap(ourdisplay, XkbAllMapComponentsMask, XkbUseCoreKbd);
//    	if (!xkb) {
//      		cout << "FATAL ERROR: XkbGetMap failed" << endl;
//  	  		do_exit(1);
//    	}
//      	//if (verbosemode) cout << "Got a new map" << endl;
//     	else if (!got_xkb) {
//      		//if (verbosemode) cout << "Backup up Xkb map" << endl;
//      		got_xkb = true;
//        	if (xkb_backup == NULL) XkbFreeClientMap(xkb_backup, XkbAllMapComponentsMask, True);
//        	xkb_backup = XkbGetMap(ourdisplay, XkbAllMapComponentsMask, XkbUseCoreKbd);
//         }
		//if (verbosemode) cout << "Checking if keycode in range" << endl;
		/* check if our new keysym is valid */
		if (!XkbKeycodeInRange(xkb, keycode)) {
      		printf("The keycode %d cannot be used, as it's not between the min(%d) and max(%d) keycode of your keyboard.\n"
            	 "Please increase the 'maximum' value in /usr/X11R6/lib/X11/xkb/keycodes/xfree86, then restart X.\n",
             	keycode, xkb-> min_key_code, xkb->max_key_code);
      		continue;
    	}
    	/* assign a group to the key code */
      	//if (verbosemode) cout << "Assigning keycodes." << endl;
		types[0] = XkbOneLevelIndex;
    	if ( XkbChangeTypesOfKey(xkb, keycode, 1, XkbGroup1Mask, types, NULL) != Success ) {
      		cerr << "FATAL ERROR: XkbChangeTypesOfKey failed" << endl;
	  		do_exit();
    	}
    	/* change the keysym */
    	if ( XkbResizeKeySyms( xkb, keycode, 1 ) == NULL ) {
      		cerr << "FATAL ERROR: resize keysym failed" << endl;
	  		do_exit();
    	}
		/* assign our new keysym to the keycode */
    	*XkbKeySymsPtr(xkb,keycode) = myKbd.getKey(keycode).getKeySym();
    	/* add one key action to it */
    	if ( XkbResizeKeyActions(xkb,keycode,1) == NULL ) {
      		cerr << "FATAL ERROR: resize key action failed" << endl;
	  		do_exit();
    	}
     //cout << "Commiting changes\n";
    	commitXKBChanges(keycode);
    	commitXKBChanges(keycode);
	  	/* Anthony was right... this really needs to be called twice :)
	     	When called only once, and the program is launched when logging into an X session,
	     	one of every two keys is not functional ... (still weird thought that when launched
	     	from an xterm, calling it only once _does_ work..)
	  	*/
    	/* assign the message action to the key code */
    	(&(xkb->server->acts[ xkb->server->key_acts[keycode] ]))->msg = xma; //static_cast<const _XkbAction>(xma);
		/* commit the change back to the server. */
		bzero(&mapChangeRec, sizeof(mapChangeRec));
    	mapChangeRec.changed = XkbKeyActionsMask;
    	mapChangeRec.first_key_act = keycode;
    	mapChangeRec.num_key_acts = 1;
    	if ( XkbChangeMap(ourdisplay, xkb, &mapChangeRec) ) {
      		if (verbosemode) cout << "Xkb map change done for keycode " << keycode << endl;
    	} else {
      		cerr << "FATAL ERROR: map changed failed" << endl;
	  		do_exit();
    	}
      // remove the X error handling stuff.
        i++;
//		if (!caughtXerror)
//	  		i++;
//		else {
//	  		if (verbosemode) cout << "*** caught X error, retrying for keycode " << keycode << endl;
//	  		caughtXerror = true;
//		}
	}

  	/* Select the ActionMessage event under any circumstances */
  	if ( !XkbSelectEvents(ourdisplay, XkbUseCoreKbd, XkbActionMessageMask, XkbActionMessageMask)) {
    	cerr << "FATAL ERROR: Couldn't select desired XKB events" << endl;
    	do_exit();
  	}

}

