/***************************************************************************
                          xosdctrl.cpp  -  description
                             -------------------
    begin                : Sat Feb 22 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif
//#define HAVE_XOSD
#ifdef HAVE_XOSD_H

#include "xosdctrl.h"
#include "eakprocs.h"
#include "cmdprefs.h"
#include "xmgr.h"

int xosdCtrl::MaxAudio = 25700;
extern CmdPrefs cmdprefs;

xosdCtrl::xosdCtrl(LConfig& config) {
	ScrollAge = true;
    create(2);
    setFont(config.XOSD_font);
    setColor(config.XOSD_color);
    setTimeout(config.XOSD_timeout);
    setPos(config.XOSD_pos);
    setAlign(config.XOSD_align);
    setHorizontalOffset(config.XOSD_hoffset);
    setVerticalOffset(config.XOSD_voffset);
    setShadowOffset(config.XOSD_soffset);

    if(cmdprefs.getVerbose()) {
      cout << "config.XOSD_font = " << config.XOSD_font << endl;
      cout << "config.XOSD_color = " << config.XOSD_color << endl;
      cout << "config.XOSD_timeout = " << config.XOSD_timeout << endl;
      cout << "config.XOSD_pos = " << config.XOSD_pos << endl;
      cout << "config.XOSD_align = " << config.XOSD_align << endl;
      cout << "config.XOSD_hoffset = " << config.XOSD_hoffset << endl;
      cout << "config.XOSD_voffset = " << config.XOSD_voffset << endl;
      cout << "config.XOSD_soffset = " << config.XOSD_soffset << endl;
    }
    
	Forcewait=false;
	ScreenLine = 0;
    /* Not really needed, but at least we aren't throwing around an unknown value */
    old_age.tv_sec=0;

    if (ScrollAge)
      gettimeofday(&old_age,0);
}

xosdCtrl::xosdCtrl(string font, string color, int delay, xosd_pos pos, xosd_align align, int hoffset, int voffset, int soffset, int lines) {
	ScrollAge = true;
    create(lines);
    setFont(font);
    setColor(color);
    setTimeout(delay);
    setPos(pos);
    setAlign(align);
    setHorizontalOffset(hoffset);
    setVerticalOffset(voffset);
    setShadowOffset(soffset);
    Forcewait=false;
    ScreenLine = 0;
    /* Not really needed, but at least we aren't throwing around an unknown value */
    old_age.tv_sec=0;

    if (ScrollAge)
    	gettimeofday(&old_age,0);

}

xosdCtrl::xosdCtrl(){
/* Set my defaults */
	ScrollAge = true;
    create(2);
    setFont(XOSD_FONT);
    setColor(XOSD_COLOR);
    setTimeout(XOSD_TIMEOUT);
    setPos(XOSD_POS);
    setAlign(XOSD_ALIGN);
    setHorizontalOffset(XOSD_HOFFSET);
    setVerticalOffset(XOSD_VOFFSET);
    setShadowOffset(XOSD_SOFFSET);
	Forcewait=false;
	ScreenLine = 0;
    /* Not really needed, but at least we aren't throwing around an unknown value */
    old_age.tv_sec=0;

    if (ScrollAge)
      gettimeofday(&old_age,0);

}
xosdCtrl::~xosdCtrl(){
//	cleanup();
}
void xosdCtrl::create(int lines) {
	Lines=lines;
	osd = xosd_create(Lines);
    /** This can happen in the situations where LANG is an asian character set */
    if (!osd) {
      cerr << "Error initializing osd: " << xosd_error << endl;
      // Attempt to instantiate an X manager object. If we cannot connect to X, then it will
      Xmgr tmpX;
      if (!tmpX.getDisplay()) {
        cerr << "Could not get the display" << endl;
        
      }
    }
   /* Not really needed, but at least we aren't throwing around an unknown value */
    old_age.tv_sec=0;

    if (ScrollAge)
    	gettimeofday(&old_age,0);
}
void xosdCtrl::init(LConfig& config){
    create(Lines);
    setFont(config.XOSD_font);
    setColor(config.XOSD_color);
    setTimeout(config.XOSD_timeout);
    setPos(config.XOSD_pos);
    setAlign(config.XOSD_align);
    setHorizontalOffset(config.XOSD_hoffset);
    setVerticalOffset(config.XOSD_voffset);
    setShadowOffset(config.XOSD_soffset);
   /* Not really needed, but at least we aren't throwing around an unknown value */
    old_age.tv_sec=0;

    if (ScrollAge)
    	gettimeofday(&old_age,0);

}
void xosdCtrl::init(){
/* Set my defaults */
    create(Lines);
    setFont(Font);
    setColor(Color);
    setTimeout(Timeout);
	setPos(Pos);
	setAlign(Align);
    setHorizontalOffset(HorizontalOffset);
    setVerticalOffset(VerticalOffset);
    setShadowOffset(ShadowOffset);
   /* Not really needed, but at least we aren't throwing around an unknown value */
    old_age.tv_sec=0;

    if (ScrollAge)
    	gettimeofday(&old_age,0);
}

void xosdCtrl::cleanup() {
   scroll();
   if (osd) {
  		xosd_destroy (osd);
    	osd = NULL;
    }
}

void xosdCtrl::agenscroll(void) {
  /* Should we age the display? */
  if (ScrollAge)
  {
//    cout << "agenscroll\n";
     gettimeofday(&new_age,0);
     if ((new_age.tv_sec - old_age.tv_sec) > ScrollAge)
     {
        scroll();
        ScreenLine=0;
     }
  }

  if (osd) {
  	if (ScreenLine >= xosd_get_number_lines(osd))
  	{
     		scroll(1);
     		ScreenLine = xosd_get_number_lines(osd)-1;
  	}
  		if (Forcewait && xosd_is_onscreen(osd)) {
     		xosd_wait_until_no_display(osd);
  	}
  }
}

void xosdCtrl::age(void) {
  old_age.tv_sec = new_age.tv_sec;
}

/* Modify the volume display output */
void xosdCtrl::volume(int volume) {
  float volcalc;
  bool verbosemode = cmdprefs.getVerbose();
  scroll();
  if (verbosemode) cout << "volume is: " << volume << endl;
  if (volume != -2 ) {
     if (volume == -1) {
        if (osd)
           xosd_display (osd, ScreenLine++, XOSD_string, "Volume Muted");
				//agenscroll();
     }
     else {
		volcalc = (float)volume;
		volcalc /= (float)MaxAudio;
		volcalc *= 100.00;
		volume = (int) volcalc;
		if (osd) {
			xosd_display (osd, 0, XOSD_string, "Volume");
			xosd_display (osd, 1, XOSD_percentage, volume);
		//agenscroll();
		}
     }
  }

}
/* Display an onscreen command */
void xosdCtrl::show(string command) {
	if (osd) {
		scroll();
	  	xosd_display (osd, ScreenLine++, XOSD_string, command.c_str());
  		//agenscroll();
	}
}

/** Read property of string font. */
const string& xosdCtrl::getFont(){
	return Font;
}
/** Write property of string font. */
void xosdCtrl::setFont( const string& _newVal){
	Font = _newVal;
	if (osd) {
		char *font =  (char *)malloc((strlen(Font.c_str())+1)*sizeof(char));
		strcpy(font, Font.c_str());
		xosd_set_font (osd, font);
		free(font);
	}
}
/** Read property of string color. */
const string& xosdCtrl::getColor(){
	return Color;
}
/** Write property of string color. */
void xosdCtrl::setColor( const string& _newVal){
	Color = _newVal;
	if (osd) {
		string tmp = '#' + Color;
		char *color =  (char *)malloc((strlen(tmp.c_str())+1)*sizeof(char));
		strcpy(color, tmp.c_str());
		xosd_set_colour (osd, color);
		free(color);
	}
}
/** Read property of int delay. */
const int& xosdCtrl::getTimeout(){
	return Timeout;
}
/** Write property of int delay. */
void xosdCtrl::setTimeout( const int& _newVal){
	Timeout = _newVal;
	if (osd)
		xosd_set_timeout (osd, Timeout);
}
/** Read property of int Forcewait. */
const bool& xosdCtrl::getForcewait(){
	return Forcewait;
}
/** Write property of int Forcewait. */
void xosdCtrl::setForcewait( const bool& _newVal){
	Forcewait = _newVal;
}
/** Read property of xosd_pos Pos. */
const xosd_pos& xosdCtrl::getPos(){
	return Pos;
}
/** Write property of xosd_pos Pos. */
void xosdCtrl::setPos( const xosd_pos& _newVal){
	Pos = _newVal;
	if (osd)
		xosd_set_pos (osd, Pos);
}
/** Read property of int Offset. */
const int& xosdCtrl::getHorizontalOffset(){
	return HorizontalOffset;
}
/** Write property of int Offset. */
void xosdCtrl::setHorizontalOffset( const int& _newVal){
	HorizontalOffset = _newVal;
	if (osd)
		xosd_set_horizontal_offset (osd, HorizontalOffset);
}
/** Read property of int Offset. */
const int& xosdCtrl::getVerticalOffset(){
	return VerticalOffset;
}
/** Write property of int Offset. */
void xosdCtrl::setVerticalOffset( const int& _newVal){
	VerticalOffset = _newVal;
	if (osd)
		xosd_set_vertical_offset (osd, VerticalOffset);
}
/** Read property of int Shadow. */
const int& xosdCtrl::getShadowOffset(){
	return ShadowOffset;
}
/** Write property of int Shadow. */
void xosdCtrl::setShadowOffset( const int& _newVal){
	ShadowOffset = _newVal;
	if (osd)
		xosd_set_shadow_offset (osd, ShadowOffset);
}
/** Read property of int ScrollAge. */
const bool& xosdCtrl::getScrollAge(){
	return ScrollAge;
}
/** Write property of int ScrollAge. */
void xosdCtrl::setScrollAge( const bool& _newVal){
	ScrollAge = _newVal;

    if (ScrollAge)
      gettimeofday(&old_age,0);  
}
/** Read property of int ScreenLine. */
const int& xosdCtrl::getScreenLine(){
	return ScreenLine;
}
/** Write property of int ScreenLine. */
void xosdCtrl::setScreenLine( const int& _newVal){
	ScreenLine = _newVal;
}
/** Read property of int Lines. */
const int& xosdCtrl::getLines(){
	return Lines;
}
/** Write property of int Lines. */
void xosdCtrl::setLines( const int& _newVal){
	Lines = _newVal;
}
/** Read property of static int MaxAudio. */
const int& xosdCtrl::getMaxAudio(){
	return MaxAudio;
}
/** Write property of static int MaxAudio. */
void xosdCtrl::setMaxAudio( const int& _newVal){
	MaxAudio = _newVal;
}
const xosd_align& xosdCtrl::getAlign(){
	return Align;
}
/** Write property of xosd_align Align. */
void xosdCtrl::setAlign( const xosd_align& _newVal){
	Align = _newVal;
	if (osd)
		xosd_set_align (osd, Align);
}
void xosdCtrl::show() {
	if (osd)
		xosd_show (osd);
}
void xosdCtrl::hide() {
	if (osd) 
		xosd_hide (osd);
}
/** Read property of xosd_align Align. */
/** Scroll the display lines number of lines */
void xosdCtrl::scroll(int lines){
	if (osd) {
		if (lines != 0)
			xosd_scroll(osd,lines);
		else
			xosd_scroll(osd,xosd_get_number_lines(osd));
	}
}
#endif
