/***************************************************************************
                          xosdctrl.h  -  description
                             -------------------
    begin                : Sat Feb 22 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#ifdef HAVE_XOSD

#ifndef XOSDCTRL_H
#define XOSDCTRL_H

extern "C" {
#include <xosd.h>
#include <sys/time.h>
}

#include <string>
#include <iostream>
#include "cmdprefs.h"
#include "lconfig.h"

using namespace std;
/**Controls functioning of XOSD.
  *@author Sheldon Lee Wen
  */

class xosdCtrl {
public: 
	xosdCtrl();
    xosdCtrl(LConfig& config);
    xosdCtrl(string font, string color, int delay, xosd_pos pos, xosd_align align, int hoffset, int voffset, int soffset, int lines);
	~xosdCtrl();
	/** This creates the osd object */
   	void create(int lines);
    /** Reinitilize the object */
    void init(LConfig& config);
    /** Reinitilize the object */
    void init();
	/* Cleanup xosd */
	void cleanup();
	/* Age the display and scroll */
	void agenscroll();
	/* Age the display */
	void age();
	/* Show the volume */
	void volume(int volume);
	/* Show a command */
	void show(string command);
  	/** Write property of string font. */
  	void setFont( const string& _newVal);
  	/** Read property of string font. */
  	const string& getFont();
  	/** Write property of string color. */
  	void setColor( const string& _newVal);
  	/** Read property of string color. */
  	const string& getColor();
  	/** Write property of int Timeout. */
  	void setTimeout( const int& _newVal);
  	/** Read property of int Timeout. */
  	const int& getTimeout();
  	/** Write property of int Forcewait. */
  	void setForcewait( const bool& _newVal);
  	/** Read property of int Forcewait. */
  	const bool& getForcewait();
  	/** Write property of xosd_pos Pos. */
  	void setPos( const xosd_pos& _newVal);
  	/** Read property of xosd_pos Pos. */
  	const xosd_pos& getPos();
  	/** Write property of xosd_align Align. */
  	void setAlign( const xosd_align& _newVal);
  	/** Read property of xosd_align Align. */
  	const xosd_align& getAlign();
  	/** Write property of int HorizontalOffset. */
  	void setHorizontalOffset( const int& _newVal);
  	/** Read property of int HorizontalOffset. */
  	const int& getHorizontalOffset();
  	/** Write property of int HorizontalOffset. */
  	void setVerticalOffset( const int& _newVal);
  	/** Read property of int HorizontalOffset. */
  	const int& getVerticalOffset();
  	/** Write property of int Shadow. */
  	void setShadowOffset( const int& _newVal);
  	/** Read property of int Shadow. */
  	const int& getShadowOffset();
  	/** Write property of int ScrollAge. */
  	void setScrollAge( const bool& _newVal);
  	/** Read property of int ScrollAge. */
  	const bool& getScrollAge();
  	/** Write property of int ScreenLine. */
  	void setScreenLine( const int& _newVal);
  	/** Read property of int ScreenLine. */
  	const int& getScreenLine();
  	/** Write property of int Lines. */
  	void setLines( const int& _newVal);
  	/** Read property of int Lines. */
  	const int& getLines();
  	/** Write property of static int MaxAudio. */
  	void setMaxAudio( const int& _newVal);
  	/** Read property of static int MaxAudio. */
  	const static int& getMaxAudio();
    void show();
    void hide();
    /** Scroll the display lines number of lines */
    void scroll(int lines = 0);
    inline bool enabled() { return (osd != NULL); }
private:
	xosd *osd;
	struct timeval old_age,new_age;
  	/** Font typeface to use */
  	string Font;
  	/** Font color to use */
  	string Color;
  	/** Delay for display */
  	int Timeout;
  	/** Are we forced to wait for the display to clear? */
  	bool Forcewait;
  	/** Position of the screen in which to display our text. */
  	xosd_pos Pos;
  	/** Show a shadow or not? */
  	int ShadowOffset;
  	/** Number of lines on the screen to use. */
  	int ScreenLine;
  	/** Number of lines of display text. */
  	int Lines;
  	/** Screen offset */
  	int HorizontalOffset, VerticalOffset;
  	/** Age and scroll the display? */
  	bool ScrollAge;
  	/** The maximum audio value. */
  	static int MaxAudio;
  	/** Set the screen alignment of xosd. */
  	xosd_align Align;
    /** Did xosd initialize correctly? */
    bool isenabled;
};

#endif
// endif HAVE_XOSD
#endif
